@@ -25,7 +25,7 @@
 #include "ex.h"
 #include "lst.h"
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "pynac-config.h"
 #endif
 #include "tostring.h"
 
@@ -191,7 +191,7 @@ static unsigned read_unsigned(std::istream &is)
 		b = b2;
 		ret |= (b & 0x7f) << shift;
 		shift += 7;
-	} while (b & 0x80);
+	} while ((b & 0x80) != 0);
 	return ret;
 }
 
@@ -216,7 +216,7 @@ std::ostream &operator<<(std::ostream &os, const archive &ar)
 	os.put('A');
 	os.put('R');
 	os.put('C');
-	write_unsigned(os, ARCHIVE_VERSION);
+	write_unsigned(os, PYNAC_ARCHIVE_VERSION);
 
 	// Write atoms
 	unsigned num_atoms = ar.atoms.size();
@@ -264,8 +264,8 @@ std::istream &operator>>(std::istream &is, archive &ar)
 	if (c1 != 'G' || c2 != 'A' || c3 != 'R' || c4 != 'C')
 		throw (std::runtime_error("not a GiNaC archive (signature not found)"));
 	unsigned version = read_unsigned(is);
-	if (version > ARCHIVE_VERSION || version < ARCHIVE_VERSION - ARCHIVE_AGE)
-		throw (std::runtime_error("archive version " + ToString(version) + " cannot be read by this GiNaC library (which supports versions " + ToString(ARCHIVE_VERSION-ARCHIVE_AGE) + " thru " + ToString(ARCHIVE_VERSION)));
+	if (version > PYNAC_ARCHIVE_VERSION || version < PYNAC_ARCHIVE_VERSION - PYNAC_ARCHIVE_AGE)
+		throw (std::runtime_error("archive version " + ToString(version) + " cannot be read by this GiNaC library (which supports versions " + ToString(PYNAC_ARCHIVE_VERSION-PYNAC_ARCHIVE_AGE) + " thru " + ToString(PYNAC_ARCHIVE_VERSION)));
 
 	// Read atoms
 	unsigned num_atoms = read_unsigned(is);
@@ -374,7 +374,7 @@ archive_node::archive_node_cit
 
 void archive_node::add_bool(const std::string &name, bool value)
 {
-	props.push_back(property(a.atomize(name), PTYPE_BOOL, value));
+	props.push_back(property(a.atomize(name), PTYPE_BOOL, static_cast<unsigned int>(value)));
 }
 
 void archive_node::add_unsigned(const std::string &name, unsigned value)
@@ -402,7 +402,7 @@ bool archive_node::find_bool(const std::string &name, bool &ret, unsigned index)
 	for (const auto & elem : props) {
 		if (elem.type == PTYPE_BOOL && elem.name == name_atom) {
 			if (found_index == index) {
-				ret = elem.value;
+				ret = (elem.value != 0u);
 				return true;
 			}
 			found_index++;
