@@ -20,6 +20,7 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <Python.h>
 #include "py_funcs.h"
 #include "constant.h"
 #include "numeric.h"
@@ -96,6 +97,8 @@ ex constant::unarchive(const archive_node &n, lst &sym_lst)
 			return Catalan;
 		else if (s == Euler.name)
 			return Euler;
+		else if (s == NaN.name)
+			return NaN;
 		else {
 			ans = py_funcs.py_get_constant(s.c_str());
 			if (PyErr_Occurred() != nullptr) {
@@ -149,21 +152,23 @@ void constant::do_print_python_repr(const print_python_repr & c, unsigned level)
 
 bool constant::info(unsigned inf) const
 {
-	if (inf == info_flags::polynomial)
-		return true;
+        if (name == "NaN")
+                return false;
+        if (inf == info_flags::polynomial)
+                return true;
         if (inf == info_flags::inexact)
                 return false;
-	if (inf == info_flags::real)
-		return domain==domain::real or domain==domain::positive ;
-	if (inf==info_flags::positive)
-		return domain == domain::positive;
-	if (inf==info_flags::nonnegative)
-		return domain == domain::positive and evalf(0, nullptr).is_zero();
-	if (inf==info_flags::infinity) {
-		return domain == domain::infinity;
-	}
-	else
-		return inherited::info(inf);
+        if (inf == info_flags::real)
+                return domain==domain::real or domain==domain::positive ;
+        if (inf==info_flags::positive)
+                return domain == domain::positive;
+        if (inf==info_flags::nonnegative)
+                return domain == domain::positive and evalf(0, nullptr).is_zero();
+        if (inf==info_flags::infinity) {
+                return domain == domain::infinity;
+        }
+        else
+                return inherited::info(inf);
 }
 
 ex constant::evalf(int level, PyObject* parent) const
@@ -233,7 +238,7 @@ bool constant::is_equal_same_type(const basic & other) const
 
 long constant::calchash() const
 {
-	hashvalue = golden_ratio_hash((p_int)tinfo() ^ serial);
+	hashvalue = golden_ratio_hash((intptr_t)tinfo() ^ serial);
 
 	setflag(status_flags::hash_calculated);
 
