@@ -22,10 +22,10 @@
 
 #include <Python.h>
 #include "ex.h"
+#include "ex_utils.h"
 #include "symbol.h"
 #include "add.h"
 #include "mul.h"
-#include "ncmul.h"
 #include "numeric.h"
 #include "power.h"
 #include "lst.h"
@@ -321,18 +321,17 @@ bool ex::is_integer_one() const
 {
     if (!is_exactly_a<numeric>(*this))
         return false;
-    return ex_to<numeric>(*this).is_integer()
-        and ex_to<numeric>(*this).is_one();
+    const numeric& num = ex_to<numeric>(*this);
+    return num.is_integer() and num.is_one();
 }
   
 bool ex::is_integer_pmone() const
 {
     if (!is_exactly_a<numeric>(*this))
         return false;
-    numeric num = ex_to<numeric>(*this);
-    return ((ex_to<numeric>(*this).is_integer())
-        and (ex_to<numeric>(*this).is_one()
-            or ex_to<numeric>(*this).is_minus_one()));
+    const numeric& num = ex_to<numeric>(*this);
+    return num.is_integer()
+           and (num.is_one() or num.is_minus_one());
 }
 
 void ex::set_domain(unsigned d)
@@ -455,6 +454,23 @@ ex ex::sorted_op(size_t i) const
 		return bp->op(i);
 }
 
+static bool has_nonposint_power(const ex& x, const symbol& symb)
+{
+        if (is_exactly_a<power>(x)) {
+                const power& p = ex_to<power>(x);
+                if (is_exactly_a<add>(p.op(0))
+                    and has_symbol(p.op(0), symb)
+                    and (not is_exactly_a<numeric>(p.op(1))
+                    or not ex_to<numeric>(p.op(1)).is_pos_integer()))
+                return true;
+        }
+        for (size_t i=0; i<x.nops(); ++i)
+        if (has_nonposint_power(x.op(i), symb))
+                return true;
+
+	return false;
+}
+
 // Helper function: Return True if term is a monomial in symb.
 // If so, fill vec (applying map at the same time) with (coeff, expo) pairs.
 static bool match_monom(const ex& term, const symbol& symb,
@@ -541,6 +557,16 @@ static bool match_monom(const ex& term, const symbol& symb,
         return false;
 }
 
+numeric ex::degree(const ex & s) const
+{
+        return bp->degree(s);
+}
+
+numeric ex::ldegree(const ex & s) const
+{
+        return bp->ldegree(s);
+}
+
 /**
  * Return in vec a list of pairs with (coefficient, exponent) of ex
  * when interpreted as polynomial in s (with s any expression).
@@ -561,9 +587,14 @@ void ex::coefficients(const ex & s, expairvec & vec) const
         exmap submap {{s, symb}}, revmap {{symb, s}};
         ex sub = subs(submap);
 
+        if (has_nonposint_power(sub, symb)) {
+                vec.push_back(std::make_pair(*this, _ex0));
+                return;
+        }
+
         if (is_exactly_a<add>(sub)) {
                 const add& addref = ex_to<add>(sub);
-                const ex& oc = addref.op(addref.nops()+1);
+                const numeric& oc = addref.get_overall_coeff();
                 if (not oc.is_zero())
                         vec.push_back(std::make_pair(oc, _ex0));
                 for (const auto& term : addref.seq) {
@@ -608,6 +639,16 @@ void ex::coefficients(const ex & s, expairvec & vec) const
 
 }
 
+ex ex::lcoeff(const ex & s) const
+{
+        return coeff(s, degree(s));
+}
+
+ex ex::tcoeff(const ex & s) const
+{
+        return coeff(s, ldegree(s));
+}
+
 ex ex::deep_combine_fractions(ex e)
 {
         if (is_a<expairseq>(e)) {
@@ -673,7 +714,7 @@ void ex::share(const ex & other) const
  *  @see ex::ex(const basic &) */
 ptr<basic> ex::construct_from_basic(const basic & other)
 {
-	if ((other.flags & status_flags::evaluated) == 0u) {
+	if (not other.is_evaluated()) {
 
 		// The object is not yet evaluated, so call eval() to evaluate
 		// the top level. This will return either
@@ -888,43 +929,6 @@ basic & ex::construct_from_long(long i)
 	}
 }
 	
-basic & ex::construct_from_ulong(unsigned long i)
-{
-	switch (i) {  // prefer flyweights over new objects
-	case 0:
-		return *const_cast<numeric *>(_num0_p);
-	case 1:
-		return *const_cast<numeric *>(_num1_p);
-	case 2:
-		return *const_cast<numeric *>(_num2_p);
-	case 3:
-		return *const_cast<numeric *>(_num3_p);
-	case 4:
-		return *const_cast<numeric *>(_num4_p);
-	case 5:
-		return *const_cast<numeric *>(_num5_p);
-	case 6:
-		return *const_cast<numeric *>(_num6_p);
-	case 7:
-		return *const_cast<numeric *>(_num7_p);
-	case 8:
-		return *const_cast<numeric *>(_num8_p);
-	case 9:
-		return *const_cast<numeric *>(_num9_p);
-	case 10:
-		return *const_cast<numeric *>(_num10_p);
-	case 11:
-		return *const_cast<numeric *>(_num11_p);
-	case 12:
-		return *const_cast<numeric *>(_num12_p);
-	default:
-		basic *bp = new numeric(i);
-		bp->setflag(status_flags::dynallocated);
-		GINAC_ASSERT(bp->get_refcount() == 0);
-		return *bp;
-	}
-}
-	
 basic & ex::construct_from_double(double d)
 {
 	basic *bp = new numeric(d);
