@@ -30,7 +30,9 @@
 #include <iterator>
 #include <functional>
 #include <stack>
+#include <unordered_set>
 
+class CanonicalForm;
 
 namespace GiNaC {
 #ifdef _MSC_VER
@@ -69,6 +71,12 @@ class scalar_products;
 class const_iterator;
 class const_preorder_iterator;
 class const_postorder_iterator;
+class symbol;
+struct symbolhasher;
+using symbolset = std::unordered_set<symbol,symbolhasher>;
+using expairvec = std::vector<std::pair<ex,ex>>;
+using ocvector = std::vector<numeric>;
+using power_ocvector_map = std::map<ex, ocvector, GiNaC::ex_is_less>;
 
 /** Lightweight wrapper for GiNaC's symbolic objects.  It holds a pointer to
  *  the other object in order to do garbage collection by the method of
@@ -82,9 +90,9 @@ class ex {
 	template<class T> friend inline bool is_a(const ex &);
 	template<class T> friend inline bool is_exactly_a(const ex &);
 	
-	friend struct print_order;
-	friend struct print_order_mul;
-	friend struct print_order_pair;
+	friend class print_order;
+	friend class print_order_mul;
+	friend class print_order_pair;
 	// default constructor, copy constructor and assignment operator
 public:
 	ex() throw();
@@ -95,7 +103,6 @@ public:
 	ex(int i);
 	ex(unsigned int i);
 	ex(long i);
-	ex(unsigned long i);
 	ex(double const d);
 	ex(PyObject* o);
 
@@ -128,9 +135,6 @@ public:
 	ex eval(int level = 0) const { return bp->eval(level); }
 	ex evalf(int level = 0, PyObject* parent=nullptr) const 
 	{ return bp->evalf(level, parent); }
-	ex evalm() const { return bp->evalm(); }
-	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
-	ex eval_integ() const { return bp->eval_integ(); }
 
 	// printing
 	void print(const print_context & c, unsigned level = 0) const;
@@ -143,7 +147,10 @@ public:
 	// operand access
 	size_t nops() const { return bp->nops(); }
 	size_t nsymbols() const;
-	ex op(size_t i) const { return bp->op(i); }
+        bool get_first_symbol(ex &x) const;
+        symbolset symbols() const;
+        symbolset free_symbols() const;
+	const ex op(size_t i) const { return bp->op(i); }
 	ex sorted_op(size_t i) const;
 	ex operator[](const ex & index) const { return (*bp)[index]; }
 	ex operator[](size_t i) const { return (*bp)[i]; }
@@ -165,7 +172,8 @@ public:
 	bool match(const ex & pattern, lst & repl_lst) const { return bp->match(pattern, repl_lst); }
 
 	// substitutions
-	ex subs(const exmap & m, unsigned options = 0) const;
+	ex subs(const exmap & m, unsigned options = 0) const
+                { return bp->subs(m, options); }
 	ex subs(const lst & ls, const lst & lr, unsigned options = 0) const;
 	ex subs(const ex & e, unsigned options = 0) const;
 
@@ -181,11 +189,12 @@ public:
 
 	// degree/coeff
 	bool is_polynomial(const ex & vars) const;
-	int degree(const ex & s) const { return bp->degree(s); }
-	int ldegree(const ex & s) const { return bp->ldegree(s); }
-	ex coeff(const ex & s, int n = 1) const { return bp->coeff(s, n); }
-	ex lcoeff(const ex & s) const { return coeff(s, degree(s)); }
-	ex tcoeff(const ex & s) const { return coeff(s, ldegree(s)); }
+	numeric degree(const ex & s) const;
+	numeric ldegree(const ex & s) const;
+	ex coeff(const ex & s, const ex & n) const { return bp->coeff(s, n); }
+	ex lcoeff(const ex & s) const;
+	ex tcoeff(const ex & s) const;
+        void coefficients(const ex & s, expairvec & vec) const;
 
 	// expand/collect
 	ex expand(unsigned options=0) const;
@@ -194,16 +203,25 @@ public:
 	// differentiation and series expansion
 	ex diff(const symbol & s, unsigned nth = 1) const;
 	ex series(const ex & r, int order, unsigned options = 0) const;
+        void useries(flint_series_t& fp, int order) const { return bp->useries(fp, order); }
 
 	// rational functions
-	ex normal(int level = 0) const;
+	ex normal(int level = 0, bool noexpand_combined=false,
+                        bool noexpand_numer=true) const;
 	ex to_rational(exmap & repl) const;
 	ex to_rational(lst & repl_lst) const;
 	ex to_polynomial(exmap & repl) const;
 	ex to_polynomial(lst & repl_lst) const;
+#ifdef PYNAC_HAVE_LIBGIAC
+        const giac::polynome to_polynome(ex_int_map& map, exvector& revmap) const;
+#endif
+        const CanonicalForm to_canonical(ex_int_map& map,
+                        power_ocvector_map& pomap, exvector& revmap) const;
+        void collect_powers(power_ocvector_map& pomap) const;
 	ex numer() const;
 	ex denom() const;
 	ex numer_denom() const;
+        ex combine_fractions(bool deep=true) const;
 
 	// polynomial algorithms
 	ex unit(const ex &x) const;
@@ -214,11 +232,8 @@ public:
 	void unitcontprim(const ex &x, ex &u, ex &c, ex &p) const;
 	ex smod(const numeric &xi) const { return bp->smod(xi); }
 	numeric max_coefficient() const;
-
-	// indexed objects
-	exvector get_free_indices() const { return bp->get_free_indices(); }
-	ex simplify_indexed(unsigned options = 0) const;
-	ex simplify_indexed(const scalar_products & sp, unsigned options = 0) const;
+        bool is_linear(const symbol& x, ex& a, ex& b) const;
+        bool is_quadratic(const symbol& x, ex& a, ex& b, ex& c) const;
 
 	// domains
 	void set_domain(unsigned d);
@@ -227,7 +242,6 @@ public:
 	int compare(const ex & other) const;
 	bool is_equal(const ex & other) const;
 	bool is_zero() const;
-	bool is_zero_matrix() const;
         bool is_integer_one() const;
         bool is_integer_pmone() const;
 	
@@ -251,11 +265,15 @@ private:
 	static basic & construct_from_pyobject(PyObject* o);
 	static basic & construct_from_uint(unsigned int i);
 	static basic & construct_from_long(long i);
-	static basic & construct_from_ulong(unsigned long i);
 	static basic & construct_from_double(double d);
 	static ptr<basic> construct_from_string_and_lst(const std::string &s, const ex &l);
 	void makewriteable();
 	void share(const ex & other) const;
+        static ex deep_combine_fractions(ex e);
+        struct combine_map_function : public map_function {
+                ex operator()(const ex & e) override { return deep_combine_fractions(e); }
+        };
+
 
 // member variables
 
@@ -306,12 +324,6 @@ ex::ex(long i) : bp(construct_from_long(i))
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 }
 
-inline
-ex::ex(unsigned long i) : bp(construct_from_ulong(i))
-{
-	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
-}
-
 inline
 ex::ex(double const d) : bp(construct_from_double(d))
 {
@@ -633,9 +645,6 @@ inline const_postorder_iterator ex::postorder_end() const throw()
 	return const_postorder_iterator();
 }
 
-
-// utility functions
-
 /** Compare two objects of class quickly without doing a deep tree traversal.
  *  @return "true" if they are equal
  *          "false" if equality cannot be established quickly (e1 and e2 may
@@ -645,6 +654,12 @@ inline bool are_ex_trivially_equal(const ex &e1, const ex &e2)
 	return e1.bp == e2.bp;
 }
 
+
+// Make it possible to print exvectors and exmaps
+std::ostream & operator<<(std::ostream & os, const exvector & e);
+std::ostream & operator<<(std::ostream & os, const exset & e);
+std::ostream & operator<<(std::ostream & os, const exmap & e);
+
 /* Function objects for STL sort() etc. */
 struct ex_is_less : public std::binary_function<ex, ex, bool> {
 	bool operator() (const ex &lh, const ex &rh) const { return lh.compare(rh) < 0; }
@@ -662,148 +677,6 @@ struct ex_swap : public std::binary_function<ex, ex, void> {
 	void operator() (ex &lh, ex &rh) const { lh.swap(rh); }
 };
 
-
-// Make it possible to print exvectors and exmaps
-std::ostream & operator<<(std::ostream & os, const exvector & e);
-std::ostream & operator<<(std::ostream & os, const exset & e);
-std::ostream & operator<<(std::ostream & os, const exmap & e);
-
-// wrapper functions around member functions
-inline size_t nops(const ex & thisex)
-{ return thisex.nops(); }
-
-inline ex expand(const ex & thisex, unsigned options = 0)
-{ return thisex.expand(options); }
-
-inline ex conjugate(const ex & thisex)
-{ return thisex.conjugate(); }
-
-inline ex real_part(const ex & thisex)
-{ return thisex.real_part(); }
-
-inline ex imag_part(const ex & thisex)
-{ return thisex.imag_part(); }
-
-inline bool has(const ex & thisex, const ex & pattern, unsigned options = 0)
-{ return thisex.has(pattern, options); }
-
-inline bool find(const ex & thisex, const ex & pattern, lst & found)
-{ return thisex.find(pattern, found); }
-
-inline bool is_polynomial(const ex & thisex, const ex & vars)
-{ return thisex.is_polynomial(vars); }
-
-inline int degree(const ex & thisex, const ex & s)
-{ return thisex.degree(s); }
-
-inline int ldegree(const ex & thisex, const ex & s)
-{ return thisex.ldegree(s); }
-
-inline ex coeff(const ex & thisex, const ex & s, int n=1)
-{ return thisex.coeff(s, n); }
-
-inline ex numer(const ex & thisex)
-{ return thisex.numer(); }
-
-inline ex denom(const ex & thisex)
-{ return thisex.denom(); }
-
-inline ex numer_denom(const ex & thisex)
-{ return thisex.numer_denom(); }
-
-inline ex normal(const ex & thisex, int level=0)
-{ return thisex.normal(level); }
-
-inline ex to_rational(const ex & thisex, lst & repl_lst)
-{ return thisex.to_rational(repl_lst); }
-
-inline ex to_rational(const ex & thisex, exmap & repl)
-{ return thisex.to_rational(repl); }
-
-inline ex to_polynomial(const ex & thisex, exmap & repl)
-{ return thisex.to_polynomial(repl); }
-
-inline ex to_polynomial(const ex & thisex, lst & repl_lst)
-{ return thisex.to_polynomial(repl_lst); }
-
-inline ex collect(const ex & thisex, const ex & s, bool distributed = false)
-{ return thisex.collect(s, distributed); }
-
-inline ex eval(const ex & thisex, int level = 0)
-{ return thisex.eval(level); }
-
-inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=nullptr)
-{ return thisex.evalf(level, parent); }
-
-inline ex evalm(const ex & thisex)
-{ return thisex.evalm(); }
-
-inline ex eval_integ(const ex & thisex)
-{ return thisex.eval_integ(); }
-
-inline ex diff(const ex & thisex, const symbol & s, unsigned nth = 1)
-{ return thisex.diff(s, nth); }
-
-inline ex series(const ex & thisex, const ex & r, int order, unsigned options = 0)
-{ return thisex.series(r, order, options); }
-
-inline bool match(const ex & thisex, const ex & pattern, lst & repl_lst)
-{ return thisex.match(pattern, repl_lst); }
-
-inline ex simplify_indexed(const ex & thisex, unsigned options = 0)
-{ return thisex.simplify_indexed(options); }
-
-inline ex simplify_indexed(const ex & thisex, const scalar_products & sp, unsigned options = 0)
-{ return thisex.simplify_indexed(sp, options); }
-
-inline ex symmetrize(const ex & thisex)
-{ return thisex.symmetrize(); }
-
-inline ex symmetrize(const ex & thisex, const lst & l)
-{ return thisex.symmetrize(l); }
-
-inline ex antisymmetrize(const ex & thisex)
-{ return thisex.antisymmetrize(); }
-
-inline ex antisymmetrize(const ex & thisex, const lst & l)
-{ return thisex.antisymmetrize(l); }
-
-inline ex symmetrize_cyclic(const ex & thisex)
-{ return thisex.symmetrize_cyclic(); }
-
-inline ex symmetrize_cyclic(const ex & thisex, const lst & l)
-{ return thisex.symmetrize_cyclic(l); }
-
-inline ex op(const ex & thisex, size_t i)
-{ return thisex.op(i); }
-
-inline ex lhs(const ex & thisex)
-{ return thisex.lhs(); }
-
-inline ex rhs(const ex & thisex)
-{ return thisex.rhs(); }
-
-inline bool is_zero(const ex & thisex)
-{ return thisex.is_zero(); }
-
-inline void swap(ex & e1, ex & e2)
-{ e1.swap(e2); }
-
-inline ex ex::subs(const exmap & m, unsigned options) const
-{
-	return bp->subs(m, options);
-}
-
-inline ex subs(const ex & thisex, const exmap & m, unsigned options = 0)
-{ return thisex.subs(m, options); }
-
-inline ex subs(const ex & thisex, const lst & ls, const lst & lr, unsigned options = 0)
-{ return thisex.subs(ls, lr, options); }
-
-inline ex subs(const ex & thisex, const ex & e, unsigned options = 0)
-{ return thisex.subs(e, options); }
-
-
 /* Convert function pointer to function object suitable for map(). */
 class pointer_to_map_function : public map_function {
 protected:
@@ -931,33 +804,7 @@ inline const T &ex_to(const ex &e)
 	return static_cast<const T &>(*e.bp);
 }
 
+
 } // namespace GiNaC
 
-
-// Specializations of Standard Library algorithms
-namespace std {
-
-/** Specialization of std::swap() for ex objects. */
-template <>
-inline void swap(GiNaC::ex &a, GiNaC::ex &b)
-{
-	a.swap(b);
-}
-
-/** Specialization of std::iter_swap() for vector<ex> iterators. */
-template <>
-inline void iter_swap(vector<GiNaC::ex>::iterator i1, vector<GiNaC::ex>::iterator i2)
-{
-	i1->swap(*i2);
-}
-
-/** Specialization of std::iter_swap() for list<ex> iterators. */
-template <>
-inline void iter_swap(list<GiNaC::ex>::iterator i1, list<GiNaC::ex>::iterator i2)
-{
-	i1->swap(*i2);
-}
-
-} // namespace std
-
 #endif // ndef __GINAC_EX_H__
