@@ -22,6 +22,7 @@
 
 #include "inifcns.h"
 #include "ex.h"
+#include "ex_utils.h"
 #include "constant.h"
 #include "lst.h"
 #include "fderivative.h"
@@ -33,6 +34,7 @@
 #include "symbol.h"
 #include "symmetry.h"
 #include "utils.h"
+#include "wildcard.h"
 
 #include <vector>
 #include <stdexcept>
@@ -394,9 +396,8 @@ static ex unit_step_eval(const ex & arg)
 	if (is_exactly_a<numeric>(arg))
 		return unit_step_evalf(arg, nullptr);
 	
-	else if (is_exactly_a<mul>(arg) &&
-	         is_exactly_a<numeric>(arg.op(arg.nops()-1))) {
-		numeric oc = ex_to<numeric>(arg.op(arg.nops()-1));
+	else if (is_exactly_a<mul>(arg)) {
+		const numeric& oc = ex_to<mul>(arg).get_overall_coeff();
 		if (oc.is_real()) {
 			if (oc > 0)
 				// step(42*x) -> step(x)
@@ -479,9 +480,8 @@ static ex heaviside_eval(const ex & arg)
 	if (is_exactly_a<numeric>(arg))
 		return heaviside_evalf(arg, nullptr);
 	
-	else if (is_exactly_a<mul>(arg) &&
-	         is_exactly_a<numeric>(arg.op(arg.nops()-1))) {
-		numeric oc = ex_to<numeric>(arg.op(arg.nops()-1));
+	else if (is_exactly_a<mul>(arg)) {
+		const numeric& oc = ex_to<mul>(arg).get_overall_coeff();
 		if (oc.is_real()) {
 			if (oc > 0)
 				// step(42*x) -> step(x)
@@ -558,9 +558,8 @@ static ex csgn_eval(const ex & arg)
 	if (is_exactly_a<numeric>(arg))
 		return csgn(ex_to<numeric>(arg));
 	
-	else if (is_exactly_a<mul>(arg) &&
-	         is_exactly_a<numeric>(arg.op(arg.nops()-1))) {
-		numeric oc = ex_to<numeric>(arg.op(arg.nops()-1));
+	else if (is_exactly_a<mul>(arg)) {
+		const numeric& oc = ex_to<mul>(arg).get_overall_coeff();
 		if (oc.is_real()) {
 			if (oc > 0)
 				// csgn(42*x) -> csgn(x)
@@ -746,8 +745,7 @@ static ex Order_eval(const ex & x)
 	} else if (is_exactly_a<mul>(x)) {
 		const mul &m = ex_to<mul>(x);
 		// O(c*expr) -> O(expr)
-		if (is_exactly_a<numeric>(m.op(m.nops() - 1)))
-			return Order(x / m.op(m.nops() - 1)).hold();
+		return Order(x / m.get_overall_coeff()).hold();
 	}
 	return Order(x).hold();
 }
@@ -758,7 +756,8 @@ static ex Order_series(const ex & x, const relational & r, int order, unsigned o
 	epvector new_seq;
 	GINAC_ASSERT(is_a<symbol>(r.lhs()));
 	const symbol &s = ex_to<symbol>(r.lhs());
-	new_seq.push_back(expair(Order(_ex1), numeric(std::min(x.ldegree(s), order))));
+        int ldeg = x.ldegree(s).to_int();
+	new_seq.push_back(expair(Order(_ex1), numeric(std::min(ldeg, order))));
 	return pseries(r, new_seq);
 }
 
@@ -792,4 +791,78 @@ REGISTER_FUNCTION(Order, eval_func(Order_eval).
                          real_part_func(Order_real_part).
                          imag_part_func(Order_imag_part));
 
+static ex cases_eval(const ex & arg)
+{
+        if (not is_exactly_a<exprseq>(arg))
+                throw std::runtime_error("cases argument not a sequence");
+        bool default_seen = false;
+        ex deflt;
+        for (auto case_ : arg) {
+                if (not is_exactly_a<exprseq>(case_)) {
+                        if (not default_seen) {
+                                deflt = case_;
+                                default_seen = true;
+                        }
+                        continue;
+                }
+                const exprseq& seq = ex_to<exprseq>(case_);
+                if (seq.nops() == 1) {
+                        if (not default_seen) {
+                                deflt = seq[0];
+                                default_seen = true;
+                        }
+                        continue;
+                }
+                if (seq.nops() != 2)
+                        throw std::runtime_error("cases case not a pair");
+                const ex& cond = seq[0];
+                if (is_exactly_a<numeric>(cond)) {
+                        if (not cond.is_zero()
+                            and not default_seen) {
+                                deflt = seq[1];
+                                default_seen = true;
+                        }
+                        continue;
+                }
+                if (is_exactly_a<relational>(cond)) {
+                        auto res = ex_to<relational>(cond).decide();
+                        if (res == relational::result::True)
+                                return seq[1];
+                        else if (res == relational::result::False)
+                                continue;
+                        // undecidable relation
+                        return cases(arg).hold();
+                }
+                throw std::runtime_error("cases with meaningless condition");
+        }
+        if (default_seen)
+                return deflt;
+        return cases(arg).hold();
+}
+
+static ex cases_evalf(const ex & arg, PyObject* parent)
+{
+        return cases(arg).hold();
+}
+
+static ex cases_conjugate(const ex & arg)
+{
+        return cases(arg).hold();
+}
+
+static ex cases_real_part(const ex & arg)
+{
+        return cases(arg).hold();
+}
+
+static ex cases_imag_part(const ex & arg)
+{
+        return cases(arg).hold();
+}
+
+REGISTER_FUNCTION(cases, eval_func(cases_eval).
+                       evalf_func(cases_evalf).
+                       conjugate_func(cases_conjugate).
+                       real_part_func(cases_real_part).
+                       imag_part_func(cases_imag_part));
 } // namespace GiNaC
