@@ -176,13 +176,13 @@ static ex tgamma_eval(const ex & x)
 			// trap positive x==(n+1/2)
 			// tgamma(n+1/2) -> Pi^(1/2)*(1*3*..*(2*n-1))/(2^n)
 			if (two_x.is_positive()) {
-				const numeric n = ex_to<numeric>(x).sub(*_num1_2_p).to_bigint();
-				return (doublefactorial(n.mul(*_num2_p).sub(*_num1_p)).div(_num2_p->pow_intexp(n))) * sqrt(Pi);
+				long n = ex_to<numeric>(x).sub(*_num1_2_p).to_long();
+				return (doublefactorial(n+n-1).div(_num2_p->pow_intexp(n))) * sqrt(Pi);
 			} else {
 				// trap negative x==(-n+1/2)
 				// tgamma(-n+1/2) -> Pi^(1/2)*(-2)^n/(1*3*..*(2*n-1))
-				const numeric n = abs(ex_to<numeric>(x).sub(*_num1_2_p).to_bigint());
-				return _num_2_p->pow_intexp(n).div(doublefactorial(n.mul(*_num2_p).sub(*_num1_p)))*sqrt(Pi);
+				long n = std::labs(ex_to<numeric>(x).sub(*_num1_2_p).to_long());
+				return _num_2_p->pow_intexp(n).div(doublefactorial(n+n-1))*sqrt(Pi);
 			}
 		}
 		if (!ex_to<numeric>(x).is_exact())
@@ -256,6 +256,8 @@ static ex beta_evalf(const ex & x, const ex & y, PyObject* parent)
 	if (is_exactly_a<numeric>(x) && is_exactly_a<numeric>(y)) {
 		const numeric &nx = ex_to<numeric>(x);
 		const numeric &ny = ex_to<numeric>(y);
+                if (parent == nullptr)
+                        parent = CC_get();
                 return (nx+ny).rgamma(parent) * nx.tgamma(parent) * ny.tgamma(parent);
 	}
 	
@@ -391,8 +393,9 @@ static ex psi1_eval(const ex & x)
 			if (nx.is_positive()) {
 				// psi(n) -> 1 + 1/2 +...+ 1/(n-1) - Euler
 				numeric rat = 0;
-				for (numeric i(nx+(*_num_1_p)); i>0; --i)
+				for (numeric i(nx+(*_num_1_p)); i>0; --i) {
 					rat += i.inverse();
+                                }
 				return rat-Euler;
 			} else {
 				// for non-positive integers there is a pole:
@@ -404,8 +407,9 @@ static ex psi1_eval(const ex & x)
 			if (nx.is_positive()) {
 				// psi((2m+1)/2) -> 2/(2m+1) + 2/2m +...+ 2/1 - Euler - 2log(2)
 				numeric rat = 0;
-				for (numeric i = (nx+(*_num_1_p))*(*_num2_p); i>0; i-=(*_num2_p))
+				for (numeric i = (nx+(*_num_1_p))*(*_num2_p); i>0; i-=(*_num2_p)) {
 					rat += (*_num2_p)*i.inverse();
+                                }
 				return rat-Euler-_ex2*log(_ex2);
 			} else {
 				// use the recurrence relation
