@@ -25,10 +25,12 @@
 
 #include "inifcns.h"
 #include "ex.h"
+#include "ex_utils.h"
 #include "constant.h"
 #include "infinity.h"
 #include "numeric.h"
 #include "mul.h"
+#include "add.h"
 #include "power.h"
 #include "operators.h"
 #include "relational.h"
@@ -50,6 +52,29 @@ namespace GiNaC {
    computer algebra systems.   These print methods are registered
    below with each of the corresponding inverse trig function. */
 
+// helper function: returns whether the expression is a multiple of I
+static bool is_multiple_of_I(const ex & the_ex)
+{
+
+	if (is_exactly_a<numeric>(the_ex)
+	    and the_ex.real_part().is_zero())
+		return true;
+
+	if (is_exactly_a<mul>(the_ex)) {
+		for (size_t i=0; i < the_ex.nops(); ++i)
+			if (is_multiple_of_I(the_ex.op(i)))
+				return true;
+	}
+
+	if (is_exactly_a<add>(the_ex)) {
+		for (size_t i=0; i < the_ex.nops(); ++i)
+			if (!is_multiple_of_I(the_ex.op(i)))
+				return false;
+		return true;
+	}
+	return false;
+};
+
 
 //////////
 // hyperbolic sine (trigonometric function)
@@ -57,6 +82,10 @@ namespace GiNaC {
 
 static ex sinh_eval(const ex & x)
 {
+        // sinh() is odd
+        if (x.info(info_flags::negative))
+                return -sinh(-x);
+
 	if (is_exactly_a<numeric>(x)) {
 
 		// sinh(0) -> 0
@@ -66,10 +95,6 @@ static ex sinh_eval(const ex & x)
 		// sinh(float) -> float
 		if (x.info(info_flags::inexact))
 			return sinh(ex_to<numeric>(x));
-
-		// sinh() is odd
-		if (x.info(info_flags::negative))
-			return -sinh(-x);
 	}
 
 	// sinh(oo) -> oo
@@ -80,15 +105,18 @@ static ex sinh_eval(const ex & x)
 			throw (std::runtime_error("sinh_eval(): sinh(unsigned_infinity) encountered"));
 		return x;
 	}
-	
-        ex xoverpi = x/Pi;
-	if (is_exactly_a<numeric>(xoverpi) &&
-		ex_to<numeric>(xoverpi).real().is_zero())  // sinh(I*x) -> I*sin(x)
-		return I*sin(x/I);
+
+	// sinh(I*x) --> I*sin(x/I)
+	if (is_multiple_of_I(x.expand()))
+	    return I*sin(x/I);
 	
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
+                // sinh(log(x)) -> (x^2 - 1)/(2x)
+		if (is_ex_the_function(x, log))
+                        return (power(t, _ex2) - _ex1)/(_ex2*t);
+
 		// sinh(asinh(x)) -> x
 		if (is_ex_the_function(x, asinh))
 			return t;
@@ -150,8 +178,11 @@ static ex cosh_evalf(const ex & x, PyObject* parent)
 
 static ex cosh_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
-
+        // cosh() is even
+        if (x.info(info_flags::negative))
+                return cosh(-x);
+	
+        if (is_exactly_a<numeric>(x)) {
 		// cosh(0) -> 1
 		if (x.is_zero())
 			return _ex1;
@@ -159,10 +190,6 @@ static ex cosh_eval(const ex & x)
 		// cosh(float) -> float
 		if (x.info(info_flags::inexact))
 			return cosh(ex_to<numeric>(x));
-
-		// cosh() is even
-		if (x.info(info_flags::negative))
-			return cosh(-x);
 	}
 	
 	// cosh(oo) -> oo
@@ -174,14 +201,17 @@ static ex cosh_eval(const ex & x)
 		return Infinity;
 	}
 
-        ex xoverpi = x/Pi;
-	if (is_exactly_a<numeric>(xoverpi) &&
-		ex_to<numeric>(xoverpi).real().is_zero())  // cosh(I*x) -> cos(x)
+	// cosh(I*x) --> cos(x)
+	if (is_multiple_of_I(x.expand()))
 		return cos(x/I);
 	
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
+                // cosh(log(x)) -> (x + 1/x)/2
+		if (is_ex_the_function(x, log))
+                        return (power(t, _ex2) + _ex1)/(_ex2*t);
+
 		// cosh(acosh(x)) -> x
 		if (is_ex_the_function(x, acosh))
 			return t;
@@ -236,8 +266,11 @@ REGISTER_FUNCTION(cosh, eval_func(cosh_eval).
 
 static ex tanh_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        // tanh() is odd
+        if (x.info(info_flags::negative))
+                return -tanh(-x);
 
+	if (is_exactly_a<numeric>(x)) {
 		// tanh(0) -> 0
 		if (x.is_zero())
 			return _ex0;
@@ -245,10 +278,6 @@ static ex tanh_eval(const ex & x)
 		// tanh(float) -> float
 		if (x.info(info_flags::inexact))
 			return tanh(ex_to<numeric>(x));
-
-		// tanh() is odd
-		if (x.info(info_flags::negative))
-			return -tanh(-x);
 	}
 	
 	// tanh(oo) -> 1
@@ -263,14 +292,17 @@ static ex tanh_eval(const ex & x)
 		throw (std::runtime_error("tanh_eval(): tanh(unsigned_infinity) encountered"));
 	}
 		
-        ex xoverpi = x/Pi;
-	if (is_exactly_a<numeric>(xoverpi) &&
-		ex_to<numeric>(xoverpi).real().is_zero())  // tanh(I*x) -> I*tan(x);
+	// tanh(I*x) --> I*tan(x)
+	if (is_multiple_of_I(x.expand()))
 		return I*tan(x/I);
 	
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
+                // tanh(log(x)) -> (x^2 - 1)/(x^2 + 1)
+		if (is_ex_the_function(x, log))
+                        return (power(t, _ex2) - _ex1)/(power(t, _ex2) + _ex1);
+
 		// tanh(atanh(x)) -> x
 		if (is_ex_the_function(x, atanh))
 			return t;
@@ -346,8 +378,11 @@ REGISTER_FUNCTION(tanh, eval_func(tanh_eval).
 
 static ex coth_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        // coth() is odd
+        if (x.info(info_flags::negative))
+                return -coth(-x);
 
+	if (is_exactly_a<numeric>(x)) {
 		// coth(0) -> zoo
 		if (x.is_zero())
 			return UnsignedInfinity;
@@ -355,10 +390,6 @@ static ex coth_eval(const ex & x)
 		// coth(float) -> float
 		if (x.info(info_flags::inexact))
 			return tanh(ex_to<numeric>(x)).inverse();
-
-		// coth() is odd
-		if (x.info(info_flags::negative))
-			return -coth(-x);
 	}
 
 	// coth(oo) -> 1
@@ -373,25 +404,17 @@ static ex coth_eval(const ex & x)
 		throw (std::runtime_error("coth_eval(): tanh(unsigned_infinity) encountered"));
 	}
 
-        ex xoverpi = x/Pi;
-	if (is_exactly_a<numeric>(xoverpi)) {
-		numeric nxopi = ex_to<numeric>(xoverpi);
-		if (nxopi.real().is_zero()) { // coth(I*x) -> I*cot(x);
-                        numeric xoverpiI = (nxopi*2)/I;
-			if (not xoverpiI.is_integer())
-				return -I*cot(x/I);
-			else if (xoverpiI.is_odd())
-				return _ex0;
-			else
-				return UnsignedInfinity;
-		}
-		else
-			return coth(x).hold();
-	}
+	// coth(I*x) --> -I*cot(x)
+	if (is_multiple_of_I(x.expand()))
+		return -I*cot(x/I);
 
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
+                // coth(log(x)) -> (x^2 + 1)/(x^2 - 1)
+		if (is_ex_the_function(x, log))
+                        return (power(t, _ex2) + _ex1)/(power(t, _ex2) - _ex1);
+
 		// coth(acoth(x)) -> x
 		if (is_ex_the_function(x, acoth))
 			return t;
@@ -465,8 +488,11 @@ REGISTER_FUNCTION(coth, eval_func(coth_eval).
 
 static ex sech_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        // sech() is even
+        if (x.info(info_flags::negative))
+                return sech(-x);
 
+	if (is_exactly_a<numeric>(x)) {
 		// sech(0) -> 1
 		if (x.is_zero())
 			return _ex1;
@@ -474,16 +500,11 @@ static ex sech_eval(const ex & x)
 		// sech(float) -> float
 		if (x.info(info_flags::inexact))
 			return cosh(ex_to<numeric>(x)).inverse();
-
-		// sech() is even
-		if (x.info(info_flags::negative))
-			return sech(-x);
 	}
 
-        ex xoverpi = x/Pi;
-	if (is_exactly_a<numeric>(xoverpi) &&
-		ex_to<numeric>(xoverpi).real().is_zero())
-		return sec(x/I);
+	// sech(x*I) --> sec(x)
+	if (is_multiple_of_I(x.expand()))
+	    return sec(x/I);
 
 	// sech(oo) -> 0
 	// sech(-oo) -> 0
@@ -498,6 +519,10 @@ static ex sech_eval(const ex & x)
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
+                // sech(log(x)) -> 2/(x + 1/x)
+		if (is_ex_the_function(x, log))
+                        return (_ex2*t) / (power(t, _ex2) + _ex1);
+
 		// sech(asech(x)) -> x
 		if (is_ex_the_function(x, asech))
 			return t;
@@ -571,8 +596,11 @@ REGISTER_FUNCTION(sech, eval_func(sech_eval).
 
 static ex csch_eval(const ex & x)
 {
-	if (is_exactly_a<numeric>(x)) {
+        // csch() is odd
+        if (x.info(info_flags::negative))
+                return -csch(-x);
 
+	if (is_exactly_a<numeric>(x)) {
 		// csch(0) -> zoo
 		if (x.is_zero())
 			return UnsignedInfinity;
@@ -580,15 +608,10 @@ static ex csch_eval(const ex & x)
 		// csch(float) -> float
 		if (x.info(info_flags::inexact))
 			return sinh(ex_to<numeric>(x)).inverse();
-
-		// csch() is odd
-		if (x.info(info_flags::negative))
-			return -csch(-x);
 	}
 
-        ex xoverpi = x/Pi;
-	if (is_exactly_a<numeric>(xoverpi) &&
-		ex_to<numeric>(xoverpi).real().is_zero())
+	// csch(I*x) --> -I*csc(x)
+	if (is_multiple_of_I(x.expand()))
 		return -I*csc(x/I);
 
 	// csch(oo) -> 0
@@ -604,11 +627,15 @@ static ex csch_eval(const ex & x)
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
-		// sech(asech(x)) -> x
+                // csch(log(x)) -> 2/(x - 1/x)
+		if (is_ex_the_function(x, log))
+                        return (_ex2*t) / (power(t, _ex2) - _ex1);
+
+		// csch(acsch(x)) -> x
 		if (is_ex_the_function(x, acsch))
 			return t;
 
-		// sech(acosh(x)) -> 1/sqrt(x^2-1)
+		// csch(acosh(x)) -> 1/sqrt(x^2-1)
 		if (is_ex_the_function(x, asinh))
 			return power(power(t,_ex2)-_ex1,_ex_1_2);
 
