@@ -26,13 +26,17 @@ namespace GiNaC {
 
 static ex hermite_evalf(const ex& n, const ex& x, PyObject* parent)
 {
+	if (not is_exactly_a<numeric>(x)
+             or not is_exactly_a<numeric>(n))
+                return hermite(n,x).hold();
+
         // see http://dlmf.nist.gov/18.5.E13
         numeric numn = ex_to<numeric>(n);
         numeric numx = ex_to<numeric>(x);
         std::vector<numeric> numveca, numvecb;
         numveca.push_back(numn / *_num_2_p);
         numveca.push_back(*_num1_2_p + (numn / *_num_2_p));
-        return pow(numx * (*_num2_p), numn) * hypergeometric_pFq(numveca, numvecb, -pow(numx, *_num2_p).inverse(), parent);
+        return pow(numx * (*_num2_p), numn) * hypergeometric_pFq(numveca, numvecb, numx.power(-2).negative(), parent);
 }
 
 static ex hermite_eval(const ex& n, const ex& x)
@@ -43,7 +47,7 @@ static ex hermite_eval(const ex& n, const ex& x)
                         if (n.info(info_flags::integer)
                                 and n.info(info_flags::odd))
                                 return _ex0;
-                if (is_exactly_a<numeric>(n) and not numx.info(info_flags::crational))
+                if (is_exactly_a<numeric>(n) and numx.info(info_flags::inexact))
                         return hermite_evalf(n, x, nullptr);
         }
 
@@ -60,7 +64,7 @@ static ex hermite_eval(const ex& n, const ex& x)
         // T(n, k) = ((-1)^((n-k)/2))*(2^k)*n!/(k!*((n-k)/2)!) if n-k is even and >=0, else 0.
         // sequentially for all viable k. Effectively there is the recurrence
         // T(n, k) = -(k+2)*(k+1)/(2*(n-k)) * T(n, k+2), with T(n, n) = 2^n
-        numeric coeff = _num2_p->power(numn);
+        numeric coeff = _num2_p->pow_intexp(numn);
         ex sum = _ex0;
         int fac = 1;
         while (numn >= 0) {
@@ -77,7 +81,8 @@ static ex hermite_eval(const ex& n, const ex& x)
 
 static ex hermite_deriv(const ex& n, const ex & x, unsigned deriv_param)
 {
-	    GINAC_ASSERT(deriv_param==0);
+	    if (deriv_param == 0)
+                    throw std::runtime_error("derivative w.r.t. to the index is not supported yet");
 	    return _ex2 * n * hermite(n-1, x).hold();
 }
 
@@ -92,6 +97,11 @@ REGISTER_FUNCTION(hermite, eval_func(hermite_eval).
 
 static ex gegenb_evalf(const ex& n, const ex &a, const ex& x, PyObject* parent)
 {
+	if (not is_exactly_a<numeric>(x)
+             or not is_exactly_a<numeric>(a)
+             or not is_exactly_a<numeric>(n))
+                return gegenbauer(n,a,x).hold();
+
         // see http://dlmf.nist.gov/18.5.E9
         numeric numn = ex_to<numeric>(n);
         numeric numx = ex_to<numeric>(x);
@@ -108,11 +118,13 @@ static ex gegenb_evalf(const ex& n, const ex &a, const ex& x, PyObject* parent)
 
 static ex gegenb_eval(const ex& n, const ex &a, const ex& x)
 {
+	numeric numa = 1;
+
 	if (is_exactly_a<numeric>(x)) {
                 numeric numx = ex_to<numeric>(x);
                 if (is_exactly_a<numeric>(n)
 			and is_exactly_a<numeric>(a)
-			and not numx.info(info_flags::crational))
+			and numx.info(info_flags::inexact))
                         return gegenb_evalf(n, a, x, nullptr);
         }
 
@@ -128,29 +140,52 @@ static ex gegenb_eval(const ex& n, const ex &a, const ex& x)
 	if (numn.is_equal(*_num1_p))
 		return _ex2 * a * x;
 
-	if (not is_exactly_a<numeric>(a)) {
-                return _ex1 * 1/numn*(2*x*(numn + a - 1)*gegenb_eval(numn-1,a,x)
-                       - (numn + 2*a - 2)*gegenb_eval(numn-2,a,x));
-        }
+	if (is_exactly_a<numeric>(a)) {
+		numa = ex_to<numeric>(a);
+		if (numa.is_zero())
+			return _ex0; // C_n^0(x) = 0
+	}
 
-        numeric numa = ex_to<numeric>(a);
-        if (not numa.info(info_flags::rational) or numa < 0)
-                throw std::runtime_error("gegenb_eval: The parameter a must be a nonnegative rational");
+	if (not is_exactly_a<numeric>(a) or numa < 0) {
+		ex p = _ex1;
+		ex sum = _ex0;
+		ex sign = _ex1;
+		ex aa = _ex1;
+		long nn = numn.to_long();
+
+		sign = - 1;
+		for (long k=0; k<=nn/2; k++) {
+			sign *= - 1;
+			aa = a;
+			p = 1;
+
+			// rising factorial (a)_{n-k}
+			for (long i=0; i<nn-k; i++) {
+				p *= aa;
+				aa = aa + 1;
+			}
+
+			p /= numeric(k).factorial();
+			p /= numeric(nn-2*k).factorial();
+			sum += pow(2*x, nn-2*k) * p * sign;
+		}
+		return sum;
+	}
 
 	// from here on see flint2/fmpq_poly/gegenbauer_c.c
 	numeric numer = numa.numer();
 	numeric denom = numa.denom();
 	numeric t = numn.factorial();
-	numeric overall_denom = pow(denom, numn) * t;
+	numeric overall_denom = denom.pow_intexp(numn) * t;
 
-	unsigned long nn = numn.to_long();
+	long nn = numn.to_long();
 	numeric p = t / (numeric(nn/2).factorial());
 	if ((nn%2) != 0u)
 		p *= *_num2_p;
 	if ((nn&2) != 0u)
 		p = -p;
 
-	for (unsigned long k=0; k < nn-nn/2; k++) {
+	for (long k=0; k < nn-nn/2; k++) {
 		p *= numer;
 		numer += denom;
 	}
@@ -176,8 +211,19 @@ static ex gegenb_eval(const ex& n, const ex &a, const ex& x)
         return sum / overall_denom;
 }
 
+static ex gegenb_deriv(const ex& n, const ex & a, const ex & x, unsigned deriv_param)
+{
+	    if (deriv_param == 0)
+                    throw std::runtime_error("derivative w.r.t. to the index is not supported yet");
+	    if (deriv_param == 1)
+                    throw std::runtime_error("derivative w.r.t. to the second index is not supported yet");
+
+	    return _ex2 * a * gegenbauer(n-1, a+1, x).hold();
+}
+
 REGISTER_FUNCTION(gegenbauer, eval_func(gegenb_eval).
                         evalf_func(gegenb_evalf).
+                        derivative_func(gegenb_deriv).
 		        latex_name("C"));
 
 } // namespace GiNaC
