@@ -22,6 +22,7 @@
 
 #include "inifcns.h"
 #include "ex.h"
+#include "ex_utils.h"
 #include "constant.h"
 #include "infinity.h"
 #include "symbol.h"
@@ -48,6 +49,31 @@ static bool has_pi(const ex & the_ex) {
                         return true;
         return false;
 }
+
+// helper function: returns whether the expression is a multiple of I
+static bool is_multiple_of_I(const ex & the_ex)
+{
+
+	if (is_exactly_a<numeric>(the_ex)
+	    and the_ex.real_part().is_zero())
+		return true;
+
+	if (is_exactly_a<mul>(the_ex)) {
+		for (size_t i=0; i < the_ex.nops(); ++i)
+			if (is_multiple_of_I(the_ex.op(i)))
+				return true;
+	}
+
+	if (is_exactly_a<add>(the_ex)) {
+		for (size_t i=0; i < the_ex.nops(); ++i)
+			if (!is_multiple_of_I(the_ex.op(i)))
+				return false;
+		return true;
+	}
+	return false;
+};
+
+
 //////////
 // sine (trigonometric function)
 //////////
@@ -71,7 +97,7 @@ static ex sin_eval(const ex & x)
         ex x_red;
         if (has_pi(x)) {
 
-                ex coef_pi = x.coeff(Pi).expand();
+                ex coef_pi = x.coeff(Pi,_ex1).expand();
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
@@ -187,6 +213,10 @@ static ex sin_eval(const ex & x)
         else
                 x_red = x;
 
+	// simplify sin(I*x) --> I*sinh(x)
+	if (is_multiple_of_I(x_red.expand()))
+		return I*sinh(x_red/I);
+
 	if (is_exactly_a<function>(x_red)) {
 		const ex &t = x_red.op(0);
 
@@ -282,7 +312,7 @@ static ex cos_eval(const ex & x)
         ex x_red;
         if (has_pi(x)) {
 
-                ex coef_pi = x.coeff(Pi).expand();
+                ex coef_pi = x.coeff(Pi,_ex1).expand();
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
@@ -392,6 +422,10 @@ static ex cos_eval(const ex & x)
         else
                 x_red = x;
 
+	// simplify cos(I*x) --> cosh(x)
+	if (is_multiple_of_I(x_red.expand()))
+		return cosh(x_red/I);
+
 	if (is_exactly_a<function>(x_red)) {
 		const ex &t = x_red.op(0);
 
@@ -488,7 +522,7 @@ static ex tan_eval(const ex & x)
         ex x_red;
         if (has_pi(x)) {
 
-                ex coef_pi = x.coeff(Pi).expand();
+                ex coef_pi = x.coeff(Pi,_ex1).expand();
                 ex rem = _ex0;
                 if (is_exactly_a<add>(coef_pi)) {
                         for (size_t i=0; i < coef_pi.nops(); i++) {
@@ -602,6 +636,9 @@ static ex tan_eval(const ex & x)
         else
                 x_red = x;
 
+	if (is_multiple_of_I(x_red.expand()))
+		return I*tanh(x_red/I);
+
 	if (is_exactly_a<function>(x_red)) {
 		const ex &t = x_red.op(0);
 
@@ -703,6 +740,14 @@ REGISTER_FUNCTION(tan, eval_func(tan_eval).
 
 static ex cot_eval(const ex & x)
 {
+	// This should be before the tests below, since multiplying infinity
+	// with other values raises runtime_errors
+	if (x.is_zero())
+		return UnsignedInfinity;
+
+	if (is_multiple_of_I(x.expand()))
+		return -I*coth(x/I);
+
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
                    
@@ -755,7 +800,7 @@ static ex cot_eval(const ex & x)
         // Reflection at Pi/2
         const ex ExOverPi = x/Pi;
         if(is_exactly_a<numeric>(ExOverPi)) {
-		ex coef_pi = x.coeff(Pi).expand();
+		ex coef_pi = x.coeff(Pi,_ex1).expand();
 		if (is_exactly_a<numeric>(coef_pi)) {
 			const numeric c = ex_to<numeric>(coef_pi);
 		        if (c.is_rational()) {
@@ -835,6 +880,9 @@ REGISTER_FUNCTION(cot, eval_func(cot_eval).
 
 static ex sec_eval(const ex & x)
 {
+	if (is_multiple_of_I(x.expand()))
+		return sech(x/I);
+
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
@@ -948,6 +996,9 @@ REGISTER_FUNCTION(sec, eval_func(sec_eval).
 static ex csc_eval(const ex & x)
 {
 
+	if (is_multiple_of_I(x.expand()))
+		return -I*csch(x/I);
+
 	if (is_exactly_a<function>(x)) {
 		const ex &t = x.op(0);
 
