@@ -23,7 +23,6 @@
 #include "mul.h"
 #include "add.h"
 #include "operators.h"
-#include "indexed.h"
 #include "lst.h"
 #include "archive.h"
 #include "utils.h"
@@ -34,6 +33,7 @@
 #include "function.h"
 #include "inifcns.h"
 #include "order.h"
+#include "mpoly.h"
 
 #include <iostream>
 #include <vector>
@@ -73,7 +73,7 @@ mul::mul()
 mul::mul(const ex & lh, const ex & rh)
 {
 	tinfo_key = &mul::tinfo_static;
-	overall_coeff = _ex1;
+	overall_coeff = *_num1_p;
 	construct_from_2_ex(lh,rh);
 	GINAC_ASSERT(is_canonical());
 }
@@ -81,7 +81,7 @@ mul::mul(const ex & lh, const ex & rh)
 mul::mul(const exvector & v, bool do_hold)
 {
 	tinfo_key = &mul::tinfo_static;
-	overall_coeff = _ex1;
+	overall_coeff = *_num1_p;
 	construct_from_exvector(v, do_hold);
 	GINAC_ASSERT(is_canonical());
 }
@@ -89,12 +89,12 @@ mul::mul(const exvector & v, bool do_hold)
 mul::mul(const epvector & v)
 {
 	tinfo_key = &mul::tinfo_static;
-	overall_coeff = _ex1;
+	overall_coeff = *_num1_p;
 	construct_from_epvector(v);
 	GINAC_ASSERT(is_canonical());
 }
 
-mul::mul(const epvector & v, const ex & oc, bool do_index_renaming)
+mul::mul(const epvector & v, const numeric & oc, bool do_index_renaming)
 {
 	tinfo_key = &mul::tinfo_static;
 	overall_coeff = oc;
@@ -119,7 +119,7 @@ mul::mul(const ex & lh, const ex & mh, const ex & rh)
 	factors.push_back(lh);
 	factors.push_back(mh);
 	factors.push_back(rh);
-	overall_coeff = _ex1;
+	overall_coeff = *_num1_p;
 	construct_from_exvector(factors);
 	GINAC_ASSERT(is_canonical());
 }
@@ -137,6 +137,8 @@ DEFAULT_ARCHIVING(mul)
 void mul::print_overall_coeff(const ex coeff_ex, const print_context & c,
 		const char *mul_sym, bool latex) const
 {
+        if (not is_exactly_a<numeric>(coeff_ex))
+                throw std::runtime_error("mul::print_overall_coeff: can't happen");
 	const numeric & num_coeff = ex_to<numeric>(coeff_ex);
 	std::stringstream tstream;
 	std::unique_ptr<print_context> tcontext_p;
@@ -154,9 +156,9 @@ void mul::print_overall_coeff(const ex coeff_ex, const print_context & c,
 		//(coeffstr.find('/') != std::string::npos) ||
 		//(coeffstr.find('*') != std::string::npos) ||
 		//(coeffstr.find('^') != std::string::npos));
-	if (num_coeff.is_integer() &&
-                num_coeff.is_equal(*_num_1_p) &&
-                !num_coeff.is_parent_pos_char())
+	if (num_coeff.is_integer()
+                and num_coeff.is_minus_one()
+                and not num_coeff.is_parent_pos_char())
 		c.s<<"-";
 	else if (parenthesis && coeffstr[0] == '-') {
 		// We want to move the '-' out of the parenthesis if it is
@@ -169,14 +171,14 @@ void mul::print_overall_coeff(const ex coeff_ex, const print_context & c,
 		else
 			c.s<<"(";
 		tstream.str("");
-		num_coeff.mul(*_num_1_p).print(*tcontext_p, 0);
+		(-num_coeff).print(*tcontext_p, 0);
 		c.s<<tstream.str();
 		if (latex)
 			c.s<<"\\right)";
 		else
 			c.s<<")";
 		c.s << mul_sym;
-	} else if (!num_coeff.is_integer() || !num_coeff.is_equal(*_num1_p)) {
+	} else if (!num_coeff.is_integer() || !num_coeff.is_one()) {
 		if (parenthesis) {
 			if (latex)
 				c.s << "\\left(";
@@ -274,18 +276,18 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 	if (!neg_powers.empty()) {
 		// Factors with negative exponent are printed as a fraction
 		if (latex_tags) {
-			ex numer = overall_coeff.numer();
-			const numeric &coeff_numer = ex_to<numeric>(numer);
+			const numeric& numer = overall_coeff.numer();
 			bool negate = false;
-			if (coeff_numer.is_integer() && coeff_numer.is_equal(*_num_1_p) &&
-					!coeff_numer.is_parent_pos_char()) {
+			if (numer.is_integer()
+                            and numer.is_minus_one()
+                            and not numer.is_parent_pos_char()) {
 				c.s<<"-";
 				negate = true;
 				//print_numer = coeff_numer.mul(*_num_1_p);
 			} else {
 				std::stringstream tstream;
 				std::unique_ptr<print_context> tcontext_p(new print_latex(tstream, c.options));
-				coeff_numer.print(*tcontext_p, 0);
+				numer.print(*tcontext_p, 0);
 				if (tstream.peek() == '-') {
 					c.s<<"-";
 					negate = true;
@@ -295,16 +297,16 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 				}*/
 			}
 
-			numeric print_numer = negate ? coeff_numer.mul(*_num_1_p) : coeff_numer;
+			numeric print_numer = negate ? -numer : numer;
 			c.s << "\\frac{";
 			if (others.empty()) {
-				if (print_numer.is_integer() && print_numer.is_equal(*_num1_p)){
+				if (print_numer.is_integer() && print_numer.is_one()){
 					c.s<<'1';
 				} else {
 					print_numer.print(c);
 				}
 			} else {
-				if (print_numer.is_integer() && print_numer.is_equal(*_num1_p)){
+				if (print_numer.is_integer() && print_numer.is_one()){
 					mul(others).eval().print(c);
 				} else {
 					mul(print_numer,mul(others).eval()).hold().print(c);
@@ -322,7 +324,9 @@ void mul::do_print_rat_func(const print_context & c, unsigned level,
 			print_overall_coeff(overall_coeff, c,
 					others.size() == 0 ? "" : sep,
 					latex_tags);
-			if (others.empty() && overall_coeff.is_integer_pmone()) {
+			if (others.empty()
+                            and (overall_coeff.is_one()
+                                 or overall_coeff.is_minus_one())) {
 				c.s<<"1";
 			} else {
 				print_exvector(others, c, sep);
@@ -383,8 +387,8 @@ void mul::do_print_csrc(const print_csrc & c, unsigned level) const
 	if (precedence() <= level)
 		c.s << "(";
 
-	if (!overall_coeff.is_integer_one()) {
-		if (overall_coeff.is_integer_pmone()) {
+	if (not overall_coeff.is_one()) {
+		if (overall_coeff.is_minus_one()) {
 			c.s << "-";
 		}
 		else {
@@ -467,7 +471,7 @@ bool mul::info(unsigned inf) const
                         for (const auto & elem : seq)
 				if (!(recombine_pair_to_ex(elem).info(inf)))
 					return false;
-			if (overall_coeff.is_integer_one() && inf == info_flags::even)
+			if (overall_coeff.is_one() && inf == info_flags::even)
 				return true;
 			return overall_coeff.info(inf);
 		}
@@ -567,12 +571,12 @@ bool mul::is_polynomial(const ex & var) const
 	return true;
 }
 
-int mul::degree(const ex & s) const
+numeric mul::degree(const ex & s) const
 {
 	// Sum up degrees of factors
-	int deg_sum = 0;
+	numeric deg_sum(0);
         for (const auto & elem : seq)
-		if (ex_to<numeric>(elem.coeff).is_integer())
+		if (ex_to<numeric>(elem.coeff).is_real())
 			deg_sum += recombine_pair_to_ex(elem).degree(s);
 		else {
 			if (elem.rest.has(s))
@@ -581,12 +585,12 @@ int mul::degree(const ex & s) const
 	return deg_sum;
 }
 
-int mul::ldegree(const ex & s) const
+numeric mul::ldegree(const ex & s) const
 {
 	// Sum up degrees of factors
-	int deg_sum = 0;
+	numeric deg_sum(0);
         for (const auto & elem : seq)
-		if (ex_to<numeric>(elem.coeff).is_integer())
+		if (ex_to<numeric>(elem.coeff).is_real())
 			deg_sum += recombine_pair_to_ex(elem).ldegree(s);
 		else {
 			if (elem.rest.has(s))
@@ -595,12 +599,12 @@ int mul::ldegree(const ex & s) const
 	return deg_sum;
 }
 
-ex mul::coeff(const ex & s, int n) const
+ex mul::coeff(const ex & s, const ex & n) const
 {
 	exvector coeffseq;
 	coeffseq.reserve(seq.size()+1);
 	
-	if (n==0) {
+	if (n.is_zero()) {
 		// product of individual coeffs
 		// if a non-zero power of s is found, the resulting product will be 0
                 for (const auto & elem : seq)
@@ -657,7 +661,7 @@ ex mul::eval(int level) const
 	}
 #endif // def DO_GINAC_ASSERT
 	
-	if ((level == 1) and (flags & status_flags::evaluated) != 0u) {
+	if (level == 1 and is_evaluated()) {
 		GINAC_ASSERT(seq.size()>0);
 		GINAC_ASSERT(seq.size()>1 || !overall_coeff.is_integer_one());
 		return *this;
@@ -679,7 +683,7 @@ ex mul::eval(int level) const
 		if (unlikely(is_ex_the_function(i->rest, exp) and
 			     num_coeff.is_integer())) {
 			exp_count++;
-			if (exp_count>1 or not num_coeff.is_equal(*_num1_p) or
+			if (exp_count>1 or not num_coeff.is_one() or
                                 not num_coeff.is_integer())
 				return eval_exponentials();
 		}
@@ -693,15 +697,15 @@ ex mul::eval(int level) const
 	} else if (seq_size==0) {
 		// *(;c) -> c
 		return overall_coeff;
-	} else if (seq_size==1 && overall_coeff.is_integer_one() &&
-		   !ex_to<numeric>(overall_coeff).is_parent_pos_char()) {
+	} else if (seq_size==1
+                   and overall_coeff.is_one()
+		   and not overall_coeff.is_parent_pos_char()) {
 		// *(x;1) -> x
 		// except in positive characteristic: 1*(x+2) = x in F_2
 		return recombine_pair_to_ex(*(seq.begin()));
 	} else if ((seq_size==1) &&
 	           is_exactly_a<add>((*seq.begin()).rest) &&
-	           ex_to<numeric>((*seq.begin()).coeff).is_integer() &&
-	           ex_to<numeric>((*seq.begin()).coeff).is_equal(*_num1_p)) {
+	           ex_to<numeric>((*seq.begin()).coeff).is_one()) {
 		// *(+(x,y,...);c) -> +(*(x,c),*(y,c),...) (c numeric(), no powers of +())
 		const add & addref = ex_to<add>((*seq.begin()).rest);
 		epvector distrseq;
@@ -709,8 +713,7 @@ ex mul::eval(int level) const
                 for (const auto & elem : addref.seq)
 			distrseq.push_back(addref.combine_pair_with_coeff_to_pair(elem, overall_coeff));
 		const ex& x = (new add(distrseq,
-		                ex_to<numeric>(addref.overall_coeff).
-		                mul_dyn(ex_to<numeric>(overall_coeff)))
+		                addref.overall_coeff.mul(overall_coeff))
 		       )->setflag(status_flags::dynallocated | status_flags::evaluated);
                 const add & result = ex_to<add>(x);
                 if (result.seq.size()==0)
@@ -736,10 +739,16 @@ ex mul::eval(int level) const
 
 			// XXX: What is the best way to check if the polynomial is a primitive? 
 			numeric c = i->rest.integer_content();
+#ifdef DO_GINAC_ASSERT
+                        if (c.is_zero())
+                                std::cerr << i->rest << "\n" << std::flush;
+#endif
 			const numeric lead_coeff =
 				ex_to<numeric>(ex_to<add>(i->rest).\
 						lead_coeff()).div(c);
-			const bool canonicalizable = lead_coeff.is_integer();
+			const bool canonicalizable = (
+                                        lead_coeff.is_exact()
+                                        and lead_coeff.is_integer());
 
 			// The following comment is no longer true for pynac.
 			// We use the print order to determine the main variable
@@ -749,7 +758,8 @@ ex mul::eval(int level) const
 			// very unlucky event it can even loop forever). Hopefully the main
 			// variable will be the same for all terms in *this
 			const bool unit_normal = lead_coeff.is_pos_integer();
-			if (likely((c == *_num1_p) && ((! canonicalizable) || unit_normal))) {
+			if (likely(((not canonicalizable) or unit_normal)
+                                   and c.is_inexact_one() )) {
 				++i;
 				continue;
 			}
@@ -765,7 +775,7 @@ ex mul::eval(int level) const
 			}
 
 			if (! unit_normal) {
-				if(unlikely(c.get_parent_char() == 2) && c == *_num1_p) {
+				if(unlikely(c.get_parent_char() == 2) && c.is_one()) {
 					c = *_num_1_p;
 				} else {
 					c = c.mul(*_num_1_p);
@@ -779,7 +789,7 @@ ex mul::eval(int level) const
 			auto primitive = new add(addref);
 			primitive->setflag(status_flags::dynallocated);
 			primitive->clearflag(status_flags::hash_calculated);
-			primitive->overall_coeff = ex_to<numeric>(primitive->overall_coeff).div_dyn(c);
+			primitive->overall_coeff /= c;
 			primitive->seq_sorted.resize(0);
 			for (auto & elem : primitive->seq)
 				elem.coeff = ex_to<numeric>(elem.coeff).div_dyn(c);
@@ -795,9 +805,9 @@ ex mul::eval(int level) const
 				++j;
 			}
 			if (s.empty()) {
-				return ex_to<numeric>(overall_coeff).mul_dyn(oc);
+				return overall_coeff.mul(oc);
 			}
-			return (new mul(s, ex_to<numeric>(overall_coeff).mul_dyn(oc))
+			return (new mul(s, overall_coeff.mul(oc))
 			       )->setflag(status_flags::dynallocated);
 		}
 	}
@@ -829,7 +839,7 @@ ex mul::eval_exponentials() const
 	else
 		s.push_back(expair(new_exp, _ex1));
 
-	mul * result = new mul(s, ex_to<numeric>(overall_coeff).mul_dyn(oc));
+	mul * result = new mul(s, overall_coeff.mul(oc));
 	return result->setflag(status_flags::dynallocated);
 }
 
@@ -864,8 +874,9 @@ ex mul::evalf(int level, PyObject* parent) const
 	--level;
         for (const auto & elem : seq)
                s.push_back(combine_ex_with_coeff_to_pair(
-                        elem.rest.evalf(level, parent), elem.coeff));
-        return mul(s, overall_coeff.evalf(level, parent));
+                        elem.rest.evalf(level, parent),
+                        ex_to<numeric>(elem.coeff)));
+        return mul(s, ex_to<numeric>(overall_coeff.evalf(level, parent)));
 }
 
 void mul::find_real_imag(ex & rp, ex & ip) const
@@ -903,18 +914,6 @@ ex mul::imag_part() const
 	return ip;
 }
 
-ex mul::eval_ncmul(const exvector & v) const
-{
-	if (seq.empty())
-		return inherited::eval_ncmul(v);
-
-	// Find first noncommutative element and call its eval_ncmul()
-        for (const auto & elem : seq)
-		if (elem.rest.return_type() == return_types::noncommutative)
-			return elem.rest.eval_ncmul(v);
-	return inherited::eval_ncmul(v);
-}
-
 bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatches, lst & repls)
 {	
 	ex origbase;
@@ -1060,16 +1059,9 @@ retry1:
 		}
 	}
 
-	bool subsfound = false;
-        for (size_t i=0; i<subsed.size(); i++) {
-                if (subsed[i]) {
-			subsfound = true;
-			break;
-		}
-	}
-	if (!subsfound)
+        if (std::any_of(subsed.cbegin(), subsed.cend(),
+                                [](bool b) { return b; } ))
 		return subs_one_level(m, options | subs_options::algebraic);
-
 	return ((*this)/divide_by)*multiply_by;
 }
 
@@ -1095,8 +1087,9 @@ ex mul::conjugate() const
 		}
 		newepv->push_back(split_ex_to_pair(c));
 	}
-	ex x = overall_coeff.conjugate();
-	if (!newepv && are_ex_trivially_equal(x, overall_coeff)) {
+        // known to be numeric
+	const numeric& x = overall_coeff.conj();
+	if (not newepv and x.is_equal(overall_coeff)) {
 		return *this;
 	}
 	ex result = thisexpairseq(newepv ? *newepv : seq, x);
@@ -1110,10 +1103,9 @@ ex mul::conjugate() const
  *  @see ex::diff */
 ex mul::derivative(const symbol & s) const
 {
-	size_t num = seq.size();
 	exvector addseq;
-	addseq.reserve(num);
-	
+	addseq.reserve(seq.size());
+
 	// D(a*b*c) = D(a)*b*c + a*D(b)*c + a*b*D(c)
 	epvector mulseq = seq;
 	auto i = seq.begin(), end = seq.end();
@@ -1122,7 +1114,8 @@ ex mul::derivative(const symbol & s) const
 		expair ep = split_ex_to_pair(power(i->rest, i->coeff - _ex1) *
 		                             i->rest.diff(s));
 		ep.swap(*i2);
-		addseq.push_back((new mul(mulseq, overall_coeff * i->coeff))->setflag(status_flags::dynallocated));
+		addseq.push_back((new mul(mulseq,
+                     overall_coeff.mul(ex_to<numeric>(i->coeff))))->setflag(status_flags::dynallocated));
 		ep.swap(*i2);
 		++i; ++i2;
 	}
@@ -1198,12 +1191,12 @@ tinfo_t mul::return_type_tinfo() const
 	return this;
 }
 
-ex mul::thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming) const
+ex mul::thisexpairseq(const epvector & v, const numeric & oc, bool do_index_renaming) const
 {
 	return (new mul(v, oc, do_index_renaming))->setflag(status_flags::dynallocated);
 }
 
-ex mul::thisexpairseq(std::unique_ptr<epvector> vp, const ex & oc, bool do_index_renaming) const
+ex mul::thisexpairseq(std::unique_ptr<epvector> vp, const numeric & oc, bool do_index_renaming) const
 {
 	return (new mul(*vp, oc, do_index_renaming))->setflag(status_flags::dynallocated);
 }
@@ -1219,10 +1212,8 @@ expair mul::split_ex_to_pair(const ex & e) const
 }
 	
 expair mul::combine_ex_with_coeff_to_pair(const ex & e,
-                                          const ex & c) const
+                                          const numeric & c) const
 {
-        GINAC_ASSERT(is_exactly_a<numeric>(c));
-
         // First, try a common shortcut:
         if (is_exactly_a<symbol>(e))
                 return expair(e, c);
@@ -1231,21 +1222,20 @@ expair mul::combine_ex_with_coeff_to_pair(const ex & e,
 	// we create a temporary power object
 	// otherwise it would be hard to correctly evaluate
 	// expression like (4^(1/3))^(3/2)
-	if (c.is_integer_one())
+	if (c.is_one())
 		return split_ex_to_pair(e);
 
 	return split_ex_to_pair(power(e,c));
 }
 	
 expair mul::combine_pair_with_coeff_to_pair(const expair & p,
-                                            const ex & c) const
+                                            const numeric & c) const
 {
         GINAC_ASSERT(is_exactly_a<numeric>(p.coeff));
-        GINAC_ASSERT(is_exactly_a<numeric>(c));
 
         if (is_exactly_a<symbol>(p.rest))
                 return expair(p.rest, p.coeff*c);
-	if (c.is_integer_one())
+	if (c.is_one())
 		return p;
         if (p.coeff.is_integer_one())
                 return expair(p.rest, c);
@@ -1290,24 +1280,22 @@ bool mul::expair_needs_further_processing(epp it)
 	return false;
 }       
 
-ex mul::default_overall_coeff() const
+numeric mul::default_overall_coeff() const
 {
-	return _ex1;
+	return *_num1_p;
 }
 
-void mul::combine_overall_coeff(const ex & c)
+void mul::combine_overall_coeff(const numeric & c)
 {
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	GINAC_ASSERT(is_exactly_a<numeric>(c));
-	overall_coeff = ex_to<numeric>(overall_coeff).mul_dyn(ex_to<numeric>(c));
+	overall_coeff *= c;
 }
 
-void mul::combine_overall_coeff(const ex & c1, const ex & c2)
+void mul::combine_overall_coeff(const numeric & c1, const numeric & c2)
 {
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	GINAC_ASSERT(is_exactly_a<numeric>(c1));
-	GINAC_ASSERT(is_exactly_a<numeric>(c2));
-	overall_coeff = ex_to<numeric>(overall_coeff).mul_dyn(ex_to<numeric>(c1).power(ex_to<numeric>(c2)));
+	ex e = c1.power(c2);
+	if (not is_exactly_a<numeric>(e))
+                throw std::runtime_error("mul::combine_overall_coeff: can't happen");
+	overall_coeff *= ex_to<numeric>(e);
 }
 
 bool mul::can_make_flat(const expair & p) const
@@ -1347,13 +1335,6 @@ ex mul::expand(unsigned options) const
                 return *this;
         }
 
-	// do not rename indices if the object has no indices at all
-	if (((options & expand_options::expand_rename_idx) == 0u) && 
-			this->info(info_flags::has_indices))
-		options |= expand_options::expand_rename_idx;
-
-	const bool skip_idx_rename = (options & expand_options::expand_rename_idx) == 0u;
-
 	// First, expand the children
 	std::unique_ptr<epvector> expanded_seqp = expandchildren(options);
 	const epvector & expanded_seq = (expanded_seqp.get() != nullptr ? *expanded_seqp : seq);
@@ -1370,7 +1351,6 @@ ex mul::expand(unsigned options) const
 		if (is_exactly_a<add>(elem.rest) &&
 			(elem.coeff.is_integer_one())) {
 			if (is_exactly_a<add>(last_expanded)) {
-
 				// Expand a product of two sums, aggressive version.
 				// Caring for the overall coefficients in separate loops can
 				// sometimes give a performance gain of up to 15%!
@@ -1385,48 +1365,44 @@ ex mul::expand(unsigned options) const
 				const auto& add2begin = add2.seq.begin();
 				const auto& add2end   = add2.seq.end();
 				epvector distrseq;
-				distrseq.reserve(add1.seq.size()+add2.seq.size());
+				auto s = add1.seq.size()+add2.seq.size();
+                                if (s > 400) {
+//                                // the condition is probably too simple
+                                        try {
+                                        // can it be converted/expanded via Singular?
+                                        last_expanded = poly_mul_expand(last_expanded,
+                                                                        elem.rest);
+                                                continue;
+                                        }
+                                        catch (std::runtime_error) {
+                                                std::cerr << "can't happen while calling poly_mul_expand\n";
+                                        }
+                                }
 
+				distrseq.reserve(s);
 				// Multiply add2 with the overall coefficient of add1 and append it to distrseq:
 				if (!add1.overall_coeff.is_zero()) {
-					if (add1.overall_coeff.is_integer_one())
+					if (add1.overall_coeff.is_one())
 						distrseq.insert(distrseq.end(),add2begin,add2end);
 					else
                                                 for (const auto & elem2 : add2.seq)
 							distrseq.push_back(expair(elem2.rest,
-                                                                ex_to<numeric>(elem2.coeff).mul_dyn(ex_to<numeric>(add1.overall_coeff))));
+                                                                ex_to<numeric>(elem2.coeff).mul_dyn(add1.overall_coeff)));
 				}
 
 				// Multiply add1 with the overall coefficient of add2 and append it to distrseq:
 				if (!add2.overall_coeff.is_zero()) {
-					if (add2.overall_coeff.is_integer_one())
+					if (add2.overall_coeff.is_one())
 						distrseq.insert(distrseq.end(),add1begin,add1end);
 					else
                                                 for (const auto & elem1 : add1.seq)
-							distrseq.push_back(expair(elem1.rest, ex_to<numeric>(elem1.coeff).mul_dyn(ex_to<numeric>(add2.overall_coeff))));
+							distrseq.push_back(expair(elem1.rest, ex_to<numeric>(elem1.coeff).mul_dyn(add2.overall_coeff)));
 				}
 
 				// Compute the new overall coefficient and put it together:
-				ex tmp_accu = (new add(distrseq, add1.overall_coeff*add2.overall_coeff))->setflag(status_flags::dynallocated);
+				ex tmp_accu = (new add(distrseq, add1.overall_coeff.mul(add2.overall_coeff)))->setflag(status_flags::dynallocated);
 
-				exvector add1_dummy_indices, add2_dummy_indices, add_indices;
 				lst dummy_subs;
-
-				if (!skip_idx_rename) {
-                                        for (const auto & elem1 : add1.seq) {
-						add_indices = get_all_dummy_indices_safely(elem1.rest);
-						add1_dummy_indices.insert(add1_dummy_indices.end(), add_indices.begin(), add_indices.end());
-					}
-                                        for (const auto & elem2 : add2.seq) {
-						add_indices = get_all_dummy_indices_safely(elem2.rest);
-						add2_dummy_indices.insert(add2_dummy_indices.end(), add_indices.begin(), add_indices.end());
-					}
-
-					sort(add1_dummy_indices.begin(), add1_dummy_indices.end(), ex_is_less());
-					sort(add2_dummy_indices.begin(), add2_dummy_indices.end(), ex_is_less());
-					dummy_subs = rename_dummy_indices_uniquely(add1_dummy_indices, add2_dummy_indices);
-				}
-
 				// Multiply explicitly all non-numeric terms of add1 and add2:
                                 for (const auto & elem2 : add2.seq) {
 					// We really have to combine terms here in order to compactify
@@ -1434,10 +1410,7 @@ ex mul::expand(unsigned options) const
 					numeric oc(*_num0_p);
 					epvector distrseq2;
 					distrseq2.reserve(add1.seq.size());
-					const ex& i2_new = (skip_idx_rename || (dummy_subs.op(0).nops() == 0) ?
-							elem2.rest :
-							elem2.rest.subs(ex_to<lst>(dummy_subs.op(0)),
-								ex_to<lst>(dummy_subs.op(1)), subs_options::no_pattern));
+					const ex& i2_new = elem2.rest;
                                         for (const auto & elem1 : add1.seq) {
 						// Don't push_back expairs which might have a rest that evaluates to a numeric,
 						// since that would violate an invariant of expairseq:
@@ -1465,21 +1438,13 @@ ex mul::expand(unsigned options) const
 	// Now the only remaining thing to do is to multiply the factors which
 	// were not sums into the "last_expanded" sum
 	if (is_exactly_a<add>(last_expanded)) {
-		size_t n = last_expanded.nops();
 		exvector distrseq;
-		distrseq.reserve(n);
+		distrseq.reserve(last_expanded.nops());
 		exvector va;
-		if (! skip_idx_rename) {
-			va = get_all_dummy_indices_safely(mul(non_adds));
-			sort(va.begin(), va.end(), ex_is_less());
-		}
 
-		for (size_t i=0; i<n; ++i) {
+		for (const auto term_in_expanded : last_expanded) {
 			epvector factors = non_adds;
-			if (skip_idx_rename)
-				factors.push_back(split_ex_to_pair(last_expanded.op(i)));
-			else
-				factors.push_back(split_ex_to_pair(rename_dummy_indices_uniquely(va, last_expanded.op(i))));
+		        factors.push_back(split_ex_to_pair(term_in_expanded));
 			ex term = (new mul(factors, overall_coeff))->setflag(status_flags::dynallocated);
 			if (can_be_further_expanded(term)) {
 				distrseq.push_back(term.expand());
@@ -1563,7 +1528,7 @@ std::unique_ptr<epvector> mul::expandchildren(unsigned options) const
 				s->push_back(split_ex_to_pair(recombine_pair_to_ex(*cit2).expand(options)));
 				++cit2;
 			}
-			return std::move(s);
+			return s;
 		}
 		++cit;
 	}
@@ -1586,7 +1551,7 @@ ex mul::without_known_factor(const ex& f) const
 			s.push_back(elem);
 	}
 
-	mul * result = new mul(s, ex_to<numeric>(overall_coeff));
+	mul * result = new mul(s, overall_coeff);
 	return result->setflag(status_flags::dynallocated);
 }
 
