@@ -30,10 +30,10 @@
 #include "normal.h"
 #include "basic.h"
 #include "ex.h"
+#include "ex_utils.h"
 #include "add.h"
 #include "constant.h"
 #include "expairseq.h"
-#include "fail.h"
 #include "inifcns.h"
 #include "lst.h"
 #include "mul.h"
@@ -69,6 +69,7 @@ namespace GiNaC {
 // Set this to enable some statistical output for the GCD routines
 #define STATISTICS 0
 
+static symbol symbol_E;
 
 /** Compute the integer content (= GCD of all numeric coefficients) of an
  *  expanded polynomial. For a polynomial with rational coefficients, this
@@ -103,10 +104,9 @@ numeric add::integer_content() const
 		l = lcm(ex_to<numeric>(it->coeff).denom(), l);
 		it++;
 	}
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	c = gcd(ex_to<numeric>(overall_coeff).numer(), c);
-	l = lcm(ex_to<numeric>(overall_coeff).denom(), l);
-	return c/l;
+	c = gcd(overall_coeff.numer(), c);
+	l = lcm(overall_coeff.denom(), l);
+	return (c/l).abs();
 }
 
 numeric mul::integer_content() const
@@ -119,8 +119,7 @@ numeric mul::integer_content() const
 		++it;
 	}
 #endif // def DO_GINAC_ASSERT
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	return abs(ex_to<numeric>(overall_coeff));
+	return overall_coeff.abs();
 }
 
 
@@ -170,8 +169,7 @@ numeric add::max_coefficient() const
 {
 	auto it = seq.begin();
 	auto itend = seq.end();
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	numeric cur_max = abs(ex_to<numeric>(overall_coeff));
+	numeric cur_max = abs(overall_coeff);
 	while (it != itend) {
 		numeric a;
 		GINAC_ASSERT(!is_exactly_a<numeric>(it->rest));
@@ -193,8 +191,7 @@ numeric mul::max_coefficient() const
 		it++;
 	}
 #endif // def DO_GINAC_ASSERT
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	return abs(ex_to<numeric>(overall_coeff));
+	return abs(overall_coeff);
 }
 
 bool ex::is_linear(const symbol& x, ex& a, ex& b) const
@@ -256,8 +253,7 @@ ex add::smod(const numeric &xi) const
 			newseq.push_back(expair(it->rest, num_coeff));
 		it++;
 	}
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	numeric num_coeff = GiNaC::smod(ex_to<numeric>(overall_coeff), xi);
+	numeric num_coeff = GiNaC::smod(overall_coeff, xi);
 	return (new add(newseq, num_coeff))->setflag(status_flags::dynallocated);
 }
 
@@ -272,8 +268,7 @@ ex mul::smod(const numeric &xi) const
 	}
 #endif // def DO_GINAC_ASSERT
 	auto  mulcopyp = new mul(*this);
-	GINAC_ASSERT(is_exactly_a<numeric>(overall_coeff));
-	mulcopyp->overall_coeff = GiNaC::smod(ex_to<numeric>(overall_coeff),xi);
+	mulcopyp->overall_coeff = GiNaC::smod(overall_coeff,xi);
 	mulcopyp->clearflag(status_flags::evaluated);
 	mulcopyp->clearflag(status_flags::hash_calculated);
 	return mulcopyp->setflag(status_flags::dynallocated);
@@ -310,7 +305,9 @@ static ex replace_with_symbol(const ex & e, exmap & repl, exmap & rev_lookup)
 	// Otherwise create new symbol and add to list, taking care that the
 	// replacement expression doesn't itself contain symbols from repl,
 	// because subs() is not recursive
-	ex es = (new symbol)->setflag(status_flags::dynallocated);
+	symbol* sp = new symbol;
+        sp->set_domain_from_ex(e_replaced);
+        ex es = sp->setflag(status_flags::dynallocated);
 	repl.insert(std::make_pair(es, e_replaced));
 	rev_lookup.insert(std::make_pair(e_replaced, es));
 	return es;
@@ -334,7 +331,9 @@ static ex replace_with_symbol(const ex & e, exmap & repl)
 	// Otherwise create new symbol and add to list, taking care that the
 	// replacement expression doesn't itself contain symbols from repl,
 	// because subs() is not recursive
-	ex es = (new symbol)->setflag(status_flags::dynallocated);
+	symbol* sp = new symbol;
+        sp->set_domain_from_ex(e_replaced);
+        ex es = sp->setflag(status_flags::dynallocated);
 	repl.insert(std::make_pair(es, e_replaced));
 	return es;
 }
@@ -459,6 +458,21 @@ static ex frac_cancel(const ex &n, const ex &d)
 	return (new lst(num * pre_factor.numer(), den * pre_factor.denom()))->setflag(status_flags::dynallocated);
 }
 
+ex function::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options) const
+{
+        if (get_serial() == exp_SERIAL::serial) {
+                GiNaC::power p(symbol_E, op(0));
+                return p.normal(repl, rev_lookup, level, options);
+        }
+        if (level == 1)
+                return (new lst(replace_with_symbol(*this, repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
+        else if (level == -max_recursion_level)
+                throw(std::runtime_error("max recursion level reached"));
+        else {
+                normal_map_function map_normal(level - 1);
+                return (new lst(replace_with_symbol(map(map_normal), repl, rev_lookup), _ex1))->setflag(status_flags::dynallocated);
+        }
+}
 
 /** Implementation of ex::normal() for a sum. It expands terms and performs
  *  fractional addition.
@@ -474,14 +488,13 @@ ex add::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options) co
 	exvector nums, dens;
 	nums.reserve(seq.size()+1);
 	dens.reserve(seq.size()+1);
-	auto it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-		ex n = ex_to<basic>(recombine_pair_to_ex(*it)).normal(repl, rev_lookup, level-1);
+        for (const auto& pair : seq) {
+		const ex& term = recombine_pair_to_ex(pair);
+		ex n = ex_to<basic>(term).normal(repl, rev_lookup, level-1);
 		nums.push_back(n.op(0));
 		dens.push_back(n.op(1));
-		it++;
 	}
-	ex n = ex_to<numeric>(overall_coeff).normal(repl, rev_lookup, level-1);
+	ex n = overall_coeff.normal(repl, rev_lookup, level-1);
 	nums.push_back(n.op(0));
 	dens.push_back(n.op(1));
 	GINAC_ASSERT(nums.size() == dens.size());
@@ -535,14 +548,13 @@ ex mul::normal(exmap & repl, exmap & rev_lookup, int level, unsigned options) co
 	exvector num; num.reserve(seq.size());
 	exvector den; den.reserve(seq.size());
 	ex n;
-	auto it = seq.begin(), itend = seq.end();
-	while (it != itend) {
-		n = ex_to<basic>(recombine_pair_to_ex(*it)).normal(repl, rev_lookup, level-1);
+        for (const auto& pair : seq) {
+		const ex& term = recombine_pair_to_ex(pair);
+		n = ex_to<basic>(term).normal(repl, rev_lookup, level-1);
 		num.push_back(n.op(0));
 		den.push_back(n.op(1));
-		it++;
 	}
-	n = ex_to<numeric>(overall_coeff).normal(repl, rev_lookup, level-1);
+	n = overall_coeff.normal(repl, rev_lookup, level-1);
 	num.push_back(n.op(0));
 	den.push_back(n.op(1));
 
@@ -647,13 +659,13 @@ ex ex::normal(int level, bool noexpand_combined, bool noexpand_numer) const
                 options |= normal_options::no_expand_combined_numer;
         if (noexpand_numer)
                 options |= normal_options::no_expand_fraction_numer;
-        
+
 	ex e = bp->normal(repl, rev_lookup, level, options);
 	GINAC_ASSERT(is_a<lst>(e));
 
-	// Re-insert replaced symbols
-	if (!repl.empty())
-		e = e.subs(repl, subs_options::no_pattern);
+	// Re-insert replaced symbols and exp functions
+        e = e.subs(repl, subs_options::no_pattern);
+	e = e.subs(symbol_E == exp(1));
 
         // Convert {numerator, denominator} form back to fraction
         if ((options & normal_options::no_expand_fraction_numer) == 0u)
@@ -677,9 +689,11 @@ ex ex::numer() const
 
 	// Re-insert replaced symbols
 	if (repl.empty())
-		return e.op(0);
+		e = e.op(0);
 	else
-		return e.op(0).subs(repl, subs_options::no_pattern);
+		e = e.op(0).subs(repl, subs_options::no_pattern);
+	e = e.subs(symbol_E == exp(1));
+        return e;
 }
 
 /** Get denominator of an expression. If the expression is not of the normal
@@ -695,11 +709,13 @@ ex ex::denom() const
 	ex e = bp->normal(repl, rev_lookup, 0);
 	GINAC_ASSERT(is_a<lst>(e));
 
-	// Re-insert replaced symbols
+        // Re-insert replaced symbols
 	if (repl.empty())
-		return e.op(1);
+		e = e.op(1);
 	else
-		return e.op(1).subs(repl, subs_options::no_pattern);
+		e = e.op(1).subs(repl, subs_options::no_pattern);
+	e = e.subs(symbol_E == exp(1));
+        return e;
 }
 
 /** Get numerator and denominator of an expression. If the expresison is not
@@ -716,10 +732,10 @@ ex ex::numer_denom() const
 	GINAC_ASSERT(is_a<lst>(e));
 
 	// Re-insert replaced symbols
-	if (repl.empty())
-		return e;
-	else
-		return e.subs(repl, subs_options::no_pattern);
+	if (not repl.empty())
+		e = e.subs(repl, subs_options::no_pattern);
+	e = e.subs(symbol_E == exp(1));
+        return e;
 }
 
 
