@@ -20,18 +20,17 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include <Python.h>
 #include "py_funcs.h"
 #include "power.h"
 #include "expairseq.h"
 #include "add.h"
 #include "mul.h"
-#include "ncmul.h"
 #include "numeric.h"
 #include "constant.h"
 #include "infinity.h"
 #include "operators.h"
 #include "inifcns.h" // for log() in power::derivative() and exp for printing
-#include "indexed.h"
 #include "symbol.h"
 #include "lst.h"
 #include "archive.h"
@@ -307,6 +306,7 @@ bool power::info(unsigned inf) const
 		case info_flags::cinteger_polynomial:
 		case info_flags::rational_polynomial:
 		case info_flags::crational_polynomial:
+                case info_flags::integer:
 			return exponent.info(info_flags::nonnegint) &&
 			       basis.info(inf);
 		case info_flags::rational_function:
@@ -322,7 +322,8 @@ bool power::info(unsigned inf) const
 			return (flags & status_flags::expanded) != 0u;
 		case info_flags::positive:
                         if (exponent.info(info_flags::even))
-                                return basis.info(info_flags::real);
+                                return basis.info(info_flags::real)
+                                   and basis.info(info_flags::nonzero);
                         if (exponent.info(info_flags::odd))
                                 return basis.info(info_flags::positive);
 			return basis.info(info_flags::positive) && exponent.info(info_flags::real);
@@ -330,22 +331,10 @@ bool power::info(unsigned inf) const
 			return (basis.info(info_flags::positive) && exponent.info(info_flags::real)) ||
 			       (basis.info(info_flags::real) && exponent.info(info_flags::integer) && exponent.info(info_flags::even));
 		case info_flags::real:
-			return basis.info(inf) && exponent.info(info_flags::integer);
-		case info_flags::has_indices: {
-			if ((flags & status_flags::has_indices) != 0u)
-				return true;
-			else if ((flags & status_flags::has_no_indices) != 0u)
-				return false;
-			else if (basis.info(info_flags::has_indices)) {
-				setflag(status_flags::has_indices);
-				clearflag(status_flags::has_no_indices);
-				return true;
-			} else {
-				clearflag(status_flags::has_indices);
-				setflag(status_flags::has_no_indices);
-				return false;
-			}
-		}
+			return ((basis.info(inf) and
+                                 (exponent.info(info_flags::integer))) or
+                                (basis.info(info_flags::positive) and
+                                 exponent.info(inf)));
 	}
 	return inherited::info(inf);
 }
@@ -395,62 +384,54 @@ bool power::is_polynomial(const ex & var) const
 	return false;
 }
 
-int power::degree(const ex & s) const
+numeric power::degree(const ex & s) const
 {
 	if (is_equal(ex_to<basic>(s)))
 		return 1;
-	else if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_integer()) {
+	else if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_real()) {
 		if (basis.is_equal(s))
-			return ex_to<numeric>(exponent).to_int();
+			return ex_to<numeric>(exponent);
 		else
-			return basis.degree(s) * ex_to<numeric>(exponent).to_int();
+			return basis.degree(s) * ex_to<numeric>(exponent);
 	} else if (basis.has(s))
 		throw(std::runtime_error("power::degree(): undefined degree because of non-integer exponent"));
 	else
 		return 0;
 }
 
-int power::ldegree(const ex & s) const 
+numeric power::ldegree(const ex & s) const 
 {
 	if (is_equal(ex_to<basic>(s)))
 		return 1;
-	else if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_integer()) {
+	else if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_real()) {
 		if (basis.is_equal(s))
-			return ex_to<numeric>(exponent).to_int();
+			return ex_to<numeric>(exponent);
 		else
-			return basis.ldegree(s) * ex_to<numeric>(exponent).to_int();
+			return basis.ldegree(s) * ex_to<numeric>(exponent);
 	} else if (basis.has(s))
 		throw(std::runtime_error("power::ldegree(): undefined degree because of non-integer exponent"));
 	else
 		return 0;
 }
 
-ex power::coeff(const ex & s, int n) const
+ex power::coeff(const ex & s, const ex & n) const
 {
 	if (is_equal(ex_to<basic>(s)))
-		return n==1 ? _ex1 : _ex0;
+		return n.is_integer_one() ? _ex1 : _ex0;
 	else if (!basis.is_equal(s)) {
 		// basis not equal to s
-		if (n == 0)
+		if (n.is_zero())
 			return *this;
 		else
 			return _ex0;
 	} else {
 		// basis equal to s
-		if (is_exactly_a<numeric>(exponent) && ex_to<numeric>(exponent).is_integer()) {
-			// integer exponent
-			int int_exp = ex_to<numeric>(exponent).to_int();
-			if (n == int_exp)
-				return _ex1;
-			else
-				return _ex0;
-		} else {
-			// non-integer exponents are treated as zero
-			if (n == 0)
-				return *this;
-			else
-				return _ex0;
-		}
+                if (n.is_zero())
+                        return _ex0;
+                else if (exponent.is_equal(n))
+                        return _ex1;
+                else
+                        return _ex0;
 	}
 }
 
@@ -471,7 +452,7 @@ ex power::coeff(const ex & s, int n) const
  *  @param level cut-off in recursive evaluation */
 ex power::eval(int level) const
 {
-	if ((level==1) && ((flags & status_flags::evaluated) != 0u))
+	if (level == 1 and is_evaluated())
 		return *this;
 	else if (level == -max_recursion_level)
 		throw(std::runtime_error("max recursion level reached"));
@@ -487,6 +468,12 @@ ex power::eval(int level) const
 	if (is_exactly_a<numeric>(ebasis)) {
 		basis_is_numerical = true;
 		num_basis = ex_to<numeric>(ebasis);
+
+                // (negative num)^even --> (positive num)^even
+                if (num_basis.is_negative()
+                    and eexponent.info(info_flags::even)
+                    and eexponent.info(info_flags::real))
+                        return power(-num_basis, eexponent);
 	}
 	if (is_exactly_a<numeric>(eexponent)) {
 		exponent_is_numerical = true;
@@ -546,7 +533,7 @@ ex power::eval(int level) const
 		return ebasis;
 
 	// ^(0,c1) -> 0 or exception  (depending on real value of c1)
-	if (ebasis.is_zero() && exponent_is_numerical) {
+	if (exponent_is_numerical and ebasis.is_zero() ) {
 		if ((num_exponent.real()).is_zero())
 			throw (std::domain_error("power::eval(): pow(0,I) is undefined"));
 		else if ((num_exponent.real()).is_negative())
@@ -560,8 +547,15 @@ ex power::eval(int level) const
 		return _ex1;
 
 	// power of a function calculated by separate rules defined for this function
-	if (is_exactly_a<function>(ebasis))
+        if (is_exactly_a<function>(ebasis)) {
+                const function& f = ex_to<function>(ebasis);
+                if (f.get_serial() == exp_SERIAL::serial) {
+                        if (is_exactly_a<numeric>(f.op(0))
+                            or exponent_is_numerical)
+                                return exp(mul(eexponent, f.op(0)));
+                }
 		return ex_to<function>(ebasis).power(eexponent);
+        }
 
 	// Turn (x^c)^d into x^(c*d) in the case that x is positive and c is real,
         // or if d is integer (and positive to preserve fraction output).
@@ -571,6 +565,12 @@ ex power::eval(int level) const
                                           and ebasis.op(1).info(info_flags::real))))
 		return power(ebasis.op(0), ebasis.op(1) * eexponent);
 
+        // (negative oc)^even --> (positive oc)^even
+        if (is_exactly_a<mul>(ebasis)
+            and ex_to<mul>(ebasis).overall_coeff.is_negative()
+            and eexponent.info(info_flags::even)
+            and eexponent.info(info_flags::real))
+                return power(-ebasis, eexponent);
 
 	if (exponent_is_numerical) {
 		// ^(c1,c2) -> c1^c2  (c1, c2 numeric(),
@@ -581,16 +581,20 @@ ex power::eval(int level) const
 			const bool exponent_is_crational = exponent_is_rational || num_exponent.is_crational();
 			if (!basis_is_crational || !exponent_is_crational) {
 				// return a plain float
-				return (new numeric(num_basis.power(num_exponent)))->setflag(status_flags::dynallocated |
-				                                                               status_flags::evaluated |
-				                                                               status_flags::expanded);
+				ex e = num_basis.power(num_exponent);
+                                if (not is_exactly_a<numeric>(e))
+	                                return (new power(ebasis, eexponent))->setflag(status_flags::dynallocated |
+	                                               status_flags::evaluated);
+                                else
+        				return e;
 			}
 
 			if (exponent_is_rational) {
-				const numeric res = num_basis.power(num_exponent);
-				if (res.is_crational()) {
-					return res;
-				}
+                                ex e = num_basis.power(num_exponent);
+                                if (is_exactly_a<numeric>(e)
+                                                and ex_to<numeric>(e).is_crational()) {
+                                        return e;
+                                }
 			}
 			GINAC_ASSERT(!num_exponent->is_integer());  // has been handled by now
 
@@ -606,23 +610,16 @@ ex power::eval(int level) const
 				}
 				if (q.is_zero()) {  // the exponent was in the allowed range 0<(n/m)<1
 					if (num_basis.is_rational()) {
-						// call rational_power_parts
-						// for a^b return c,d such that a^b = c*d^b
-						PyObject* basis_py = num_basis.to_pyobject();
-						PyObject* exponent_py = num_exponent.to_pyobject();
-						PyObject* restuple = py_funcs.py_rational_power_parts(basis_py, exponent_py);
-						if(restuple == nullptr) {
-							throw(std::runtime_error("power::eval, error in rational_power_parts"));
-						}
-						PyObject *ppower = PyTuple_GetItem(restuple,0);
-						PyObject *newbasis = PyTuple_GetItem(restuple,1);
-						const bool ppower_equals_one = PyObject_IsTrue(PyTuple_GetItem(restuple, 2)) != 0;
+						bool ppower_equals_one;
+                                                numeric newbasis, ppower;
+                                                rational_power_parts(num_basis,
+                                                                num_exponent, 
+                                                                ppower, 
+                                                                newbasis, 
+                                                                ppower_equals_one);
 						ex result = (new power(newbasis,exponent))->setflag(status_flags::dynallocated | status_flags::evaluated);
 						if (not ppower_equals_one)
 							result = (new mul(result, ppower))->setflag(status_flags::dynallocated | status_flags::evaluated);
-						Py_DECREF(restuple);
-						Py_DECREF(basis_py);
-						Py_DECREF(exponent_py);
 						return result;
 					}
 					return this->hold();
@@ -656,11 +653,15 @@ ex power::eval(int level) const
 						|| (num_sub_exponent == *_num_1_p && num_exponent.is_positive())) {
 					return power(sub_basis,num_sub_exponent.mul(num_exponent));
 				}
+                                numeric pexp = num_sub_exponent * num_exponent;
+                                if (pexp.is_integer()
+                                    and num_sub_exponent.is_even()
+                                    and sub_basis.info(info_flags::real))
+                                        return power(abs(sub_basis), pexp);
 			}
 		}
 	
 		if (num_exponent.is_integer()) {
-                        
                         // ^(*(x,y,z),c1) -> *(x^c1,y^c1,z^c1) (c1 integer)
                         if (is_exactly_a<mul>(ebasis)) {
                                 return expand_mul(ex_to<mul>(ebasis), num_exponent, 0);
@@ -674,25 +675,28 @@ ex power::eval(int level) const
                                                         lead_coeff()).div(icont);
 
                                 const bool canonicalizable = lead_coeff.is_integer();
-                                const bool unit_normal = lead_coeff.is_pos_integer() || lead_coeff.is_parent_pos_char();
-                                if (canonicalizable && (! unit_normal))
-                                        icont = icont.mul(*_num_1_p);
+                                const add& addref = ex_to<add>(ebasis);
+                                const bool unit_normal = lead_coeff.is_positive() || lead_coeff.is_parent_pos_char();
+                                if (canonicalizable
+                                    and (! unit_normal)
+                                    and num_exponent.denom().is_one())
+                                        icont = icont.negative();
 
-                                if (canonicalizable && (icont != *_num1_p)) {
-                                        const add& addref = ex_to<add>(ebasis);
+                                if (canonicalizable
+                                    and not icont.is_one()) {
                                         auto  addp = new add(addref);
                                         addp->setflag(status_flags::dynallocated);
                                         addp->clearflag(status_flags::hash_calculated);
-                                        addp->overall_coeff = ex_to<numeric>(addp->overall_coeff).div_dyn(icont);
+                                        addp->overall_coeff = addp->overall_coeff.div_dyn(icont);
                                         addp->seq_sorted.resize(0);
                                         for (auto & elem : addp->seq)
                                                 elem.coeff = ex_to<numeric>(elem.coeff).div_dyn(icont);
-
-                                        const numeric c = icont.power(num_exponent);
-                                        if (likely(c != *_num1_p))
-                                                return (new mul(power(*addp, num_exponent), c))->setflag(status_flags::dynallocated);
-                                        else
-                                                return power(*addp, num_exponent);
+                                        const numeric c = icont.pow_intexp(num_exponent.to_long());
+                                        if (likely(not c.is_one()))
+                                                return (new mul(power(*addp, num_exponent), c))->setflag(status_flags::dynallocated |
+	                                               status_flags::evaluated);
+                                        return (new power(*addp, eexponent))->setflag(status_flags::dynallocated |
+	                                               status_flags::evaluated);
                                 }
                         }
                 }
@@ -702,12 +706,12 @@ ex power::eval(int level) const
 		if (is_exactly_a<mul>(ebasis)) {
 			GINAC_ASSERT(!num_exponent.is_integer()); // should have been handled above
 			const mul & mulref = ex_to<mul>(ebasis);
-			if (!mulref.overall_coeff.is_equal(_ex1)) {
-				const numeric & num_coeff = ex_to<numeric>(mulref.overall_coeff);
+			if (not mulref.overall_coeff.is_one()) {
+				const numeric & num_coeff = mulref.overall_coeff;
 				if (num_coeff.is_real()) {
 					if (num_coeff.is_positive()) {
 						auto mulp = new mul(mulref);
-						mulp->overall_coeff = _ex1;
+						mulp->overall_coeff = *_num1_p;
 						mulp->setflag(status_flags::dynallocated);
 						mulp->clearflag(status_flags::evaluated);
 						mulp->clearflag(status_flags::hash_calculated);
@@ -715,10 +719,10 @@ ex power::eval(int level) const
 						return (new mul(power(*mulp,exponent),
 						                power(num_coeff, num_exponent)))->setflag(status_flags::dynallocated);
 					} else {
-						GINAC_ASSERT(num_coeff.compare(*_num0_p)<0);
-						if (!num_coeff.is_equal(*_num_1_p)) {
+						GINAC_ASSERT(num_coeff.is_negative());
+						if (not num_coeff.is_minus_one()) {
 							auto mulp = new mul(mulref);
-							mulp->overall_coeff = _ex_1;
+							mulp->overall_coeff = *_num_1_p;
 							mulp->setflag(status_flags::dynallocated);
 							mulp->clearflag(status_flags::evaluated);
 							mulp->clearflag(status_flags::hash_calculated);
@@ -729,19 +733,47 @@ ex power::eval(int level) const
 					}
 				}
 			}
-		}
+                        else {
+                                // (x*y^(m/n)*z)^(r/s) ---> y^t*(x*z)^(r/s), if t integer
+                                exvector outer, inner;
+                                for (size_t i=0; i<mulref.nops(); ++i) {
+                                        const ex& fac = mulref.op(i);
+                                        ex pfac = power(fac, eexponent);
+                                        if (not is_exactly_a<power>(pfac)
+                                            or ex_to<power>(pfac).exponent.info(info_flags::integer))
+                                                outer.push_back(pfac);
+                                        else
+                                                inner.push_back(fac);
+                                }
+                                if (outer.size() > 0) {
+                                        ex outex, innex;
+                                        if (outer.size() == 1)
+                                                outex = outer[0];
+                                        else
+                                                outex = mul(outer).hold();
+                                        if (inner.size() > 0) {
+                                                if (inner.size() == 1)
+                                                        innex = inner[0];
+                                                else
+                                                        innex = mul(inner).hold();
+                                                ex p = power(innex, eexponent).hold();
+                                                return (new mul(outex, p))->setflag(status_flags::dynallocated | status_flags::evaluated);
+                                        }
+                                        return outex;
+                                }
+                        }
+                }
 	}
 
 	// Reduce x^(c/log(x)) to exp(c) if x is positive
 	if (ebasis.info(info_flags::positive)) {
 		if (eexponent.is_equal(1/log(ebasis)))
 			return exp(log(basis)*exponent);
-		else if (is_exactly_a<mul>(eexponent)) {
-			for (size_t i=0; i < eexponent.nops(); i++) {
-				if (eexponent.op(i).is_equal(1/log(ebasis)))
+		else if (is_exactly_a<mul>(eexponent) and
+			 std::any_of(eexponent.begin(), eexponent.end(),
+                                [ebasis](ex e)
+                                { return e.is_equal(1/log(ebasis)); }))
 					return exp(log(basis)*exponent);
-			}
-		}
 	}
 	
 	if (are_ex_trivially_equal(ebasis,basis) &&
@@ -826,11 +858,6 @@ ex power::subs(const exmap & m, unsigned options) const
 	return subs_one_level(m, options);
 }
 
-ex power::eval_ncmul(const exvector & v) const
-{
-	return inherited::eval_ncmul(v);
-}
-
 ex power::conjugate() const
 {
 	// conjugate(pow(x,y))==pow(conjugate(x),conjugate(y)) unless on the
@@ -920,7 +947,7 @@ ex power::derivative(const symbol & s) const
 		newseq.reserve(2);
 		newseq.push_back(expair(basis, exponent - _ex1));
 		newseq.push_back(expair(basis.diff(s), _ex1));
-		return mul(newseq, exponent);
+		return mul(newseq, ex_to<numeric>(exponent));
 	} else {
 	    // If the exponent is not a function of s, we have the following nice
 	    // looking formula.  We use this to avoid getting ugly and hard to
@@ -992,10 +1019,11 @@ ex power::expand(unsigned options) const
 
 		// take care on the numeric coefficient
 		ex coef = (possign? _ex1 : _ex_1);
-		if (m.overall_coeff.info(info_flags::positive)
-                                and not m.overall_coeff.is_integer_one())
+		if (m.overall_coeff.is_positive()
+                                and not m.overall_coeff.is_one())
 			prodseq.push_back(power(m.overall_coeff, exponent));
-		else if (m.overall_coeff.info(info_flags::negative) && m.overall_coeff != _ex_1)
+		else if (m.overall_coeff.is_negative()
+                         and not m.overall_coeff.is_minus_one())
 			prodseq.push_back(power(-m.overall_coeff, exponent));
 		else
 			coef *= m.overall_coeff;
@@ -1023,8 +1051,8 @@ ex power::expand(unsigned options) const
 			distrseq.push_back(power(expanded_basis, a.recombine_pair_to_ex(elem)));
 		
 		// Make sure that e.g. (x+y)^(2+a) expands the (x+y)^2 factor
-		if (ex_to<numeric>(a.overall_coeff).is_integer()) {
-			const numeric &num_exponent = ex_to<numeric>(a.overall_coeff);
+		if (a.overall_coeff.is_integer()) {
+			const numeric &num_exponent = a.overall_coeff;
 			int int_exponent = num_exponent.to_int();
 			if (int_exponent > 0 && is_exactly_a<add>(expanded_basis))
 				distrseq.push_back(expand_add(ex_to<add>(expanded_basis), int_exponent, options));
@@ -1193,7 +1221,11 @@ private:
 		explicit coolmulti(const std::vector<int>& partition)
 		  : head(nullptr), i(nullptr), after_i(nullptr)
 		{
-			for (unsigned n = 0; n < partition.size(); ++n) {
+                        if (partition.size() <= 0)
+                                throw std::runtime_error("can't happn in coolmuli");
+                        head = new element(partition[0], head);
+                        i = head;
+			for (unsigned n = 1; n < partition.size(); ++n) {
 				head = new element(partition[n], head);
 				if (n <= 1)
 					i = head;
@@ -1338,8 +1370,10 @@ ex power::expand_add(const add & a, long n, unsigned options) const
 	// i.e. the number of unordered arrangements of m nonnegative integers
 	// which sum up to n.  It is frequently written as C_n(m) and directly
 	// related with binomial coefficients: binomial(n+m-1,m-1).
-	size_t result_size = binomial(numeric(n+a.nops()-1), numeric(a.nops()-1)).to_long();
-	if (!a.overall_coeff.is_zero()) {
+        long anops = a.nops() - 1;
+	long result_size = binomial(numeric(n + anops),
+                        numeric(anops)).to_long();
+	if (not a.overall_coeff.is_zero()) {
 		// the result's overall_coeff is one of the terms
 		--result_size;
 	}
@@ -1361,7 +1395,7 @@ ex power::expand_add(const add & a, long n, unsigned options) const
                         if (n == k)
                                 binomial_coefficient = *_num1_p;
                         else
-                                binomial_coefficient = binomial(numeric(n), numeric(k)) * pow(ex_to<numeric>(a.overall_coeff), numeric(n-k));
+                                binomial_coefficient = binomial(numeric(n), numeric(k)) * a.overall_coeff.power(n-k);
 		}
 
 		// Multinomial expansion of power(+(x,...,z;0),k)*c^(n-k):
@@ -1397,11 +1431,11 @@ ex power::expand_add(const add & a, long n, unsigned options) const
 					} else if (the_exponent[i] == 1) {
 						// optimized
 						monomial.push_back(expair(r, _ex1));
-						if (c != *_num1_p)
+						if (not c.is_one())
 							factor = factor.mul(c);
 					} else { // general case exponent[i] > 1
 						monomial.push_back(expair(r, the_exponent[i]));
-						if (c != *_num1_p)
+						if (not c.is_one())
 							factor = factor.mul(c.power(the_exponent[i]));
 					}
 				}
@@ -1415,7 +1449,7 @@ ex power::expand_add(const add & a, long n, unsigned options) const
 	if (a.overall_coeff.is_zero()) {
 		return dynallocate<add>(std::move(result)).setflag(status_flags::expanded).eval();
 	} else {
-		return dynallocate<add>(std::move(result), ex_to<numeric>(a.overall_coeff).power(n)).setflag(status_flags::expanded).eval();
+		return dynallocate<add>(std::move(result), a.overall_coeff.power(n)).setflag(status_flags::expanded).eval();
 	}
 }
 
@@ -1439,7 +1473,8 @@ ex power::expand_add_2(const add & a, unsigned options) const
 	for (auto cit0=a.seq.begin(); cit0!=last; ++cit0) {
 		const ex & r = cit0->rest;
 		const ex & c = cit0->coeff;
-		
+		const numeric& nc = ex_to<numeric>(c);
+
 		GINAC_ASSERT(!is_exactly_a<add>(r));
 		GINAC_ASSERT(!is_exactly_a<power>(r) ||
 		             !is_exactly_a<numeric>(ex_to<power>(r).exponent) ||
@@ -1448,7 +1483,7 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		             !is_exactly_a<mul>(ex_to<power>(r).basis) ||
 		             !is_exactly_a<power>(ex_to<power>(r).basis));
 		
-		if (c.is_equal(_ex1)) {
+		if (c.is_integer_one()) {
 			if (is_exactly_a<mul>(r)) {
 				result.push_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
 				                        _ex1));
@@ -1459,10 +1494,10 @@ ex power::expand_add_2(const add & a, unsigned options) const
 		} else {
 			if (is_exactly_a<mul>(r)) {
 				result.push_back(expair(expand_mul(ex_to<mul>(r), *_num2_p, options, true),
-				                        ex_to<numeric>(c).power_dyn(*_num2_p)));
+				                        nc.pow_intexp(*_num2_p)));
 			} else {
 				result.push_back(expair(dynallocate<power>(r, _ex2),
-				                        ex_to<numeric>(c).power_dyn(*_num2_p)));
+				                        nc.pow_intexp(*_num2_p)));
 			}
 		}
 
@@ -1477,7 +1512,7 @@ ex power::expand_add_2(const add & a, unsigned options) const
 	// second part: add terms coming from overall_coeff (if != 0)
 	if (!a.overall_coeff.is_zero()) {
 		for (auto & i : a.seq)
-			result.push_back(a.combine_pair_with_coeff_to_pair(i, ex_to<numeric>(a.overall_coeff).mul_dyn(*_num2_p)));
+			result.push_back(a.combine_pair_with_coeff_to_pair(i, a.overall_coeff.mul(*_num2_p)));
 	}
 
 	GINAC_ASSERT(result.size() == result_size);
@@ -1485,7 +1520,7 @@ ex power::expand_add_2(const add & a, unsigned options) const
 	if (a.overall_coeff.is_zero()) {
 		return dynallocate<add>(std::move(result)).setflag(status_flags::expanded);
 	} else {
-		return dynallocate<add>(std::move(result), ex_to<numeric>(a.overall_coeff).power(2)).setflag(status_flags::expanded);
+		return dynallocate<add>(std::move(result), a.overall_coeff.pow_intexp(*_num2_p)).setflag(status_flags::expanded);
 	}
 }
 
@@ -1499,22 +1534,6 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 		return _ex1;
 	}
 
-	// do not bother to rename indices if there are no any.
-	if (((options & expand_options::expand_rename_idx) == 0u) 
-			&& m.info(info_flags::has_indices))
-		options |= expand_options::expand_rename_idx;
-	// Leave it to multiplication since dummy indices have to be renamed
-	if (((options & expand_options::expand_rename_idx) != 0u) &&
-		(get_all_dummy_indices(m).size() > 0) && n.is_positive()) {
-		ex result = m;
-		exvector va = get_all_dummy_indices(m);
-		sort(va.begin(), va.end(), ex_is_less());
-
-		for (int i=1; i < n.to_int(); i++)
-			result *= rename_dummy_indices_uniquely(va, m);
-		return result;
-	}
-
 	epvector distrseq;
 	distrseq.reserve(m.seq.size());
 	bool need_reexpand = false;
@@ -1529,7 +1548,7 @@ ex power::expand_mul(const mul & m, const numeric & n, unsigned options, bool fr
 		distrseq.push_back(p);
 	}
 
-	const mul & result = static_cast<const mul &>((new mul(distrseq, ex_to<numeric>(m.overall_coeff).power_dyn(n)))->setflag(status_flags::dynallocated));
+	const mul & result = static_cast<const mul &>((new mul(distrseq, m.overall_coeff.pow_intexp(n)))->setflag(status_flags::dynallocated));
 	if (need_reexpand)
 		return ex(result).expand(options);
 	if (from_expand)
