@@ -22,6 +22,8 @@
  */
 
 #include "pseries.h"
+#include "useries.h"
+#include "useries-flint.h"
 #include "add.h"
 #include "inifcns.h" // for Order function
 #include "lst.h"
@@ -30,7 +32,6 @@
 #include "relational.h"
 #include "operators.h"
 #include "symbol.h"
-#include "integral.h"
 #include "archive.h"
 #include "utils.h"
 #include "infinity.h"
@@ -280,7 +281,7 @@ const ex pseries::op(size_t i) const
 /** Return degree of highest power of the series.  This is usually the exponent
  *  of the Order term.  If s is not the expansion variable of the series, the
  *  series is examined termwise. */
-int pseries::degree(const ex &s) const
+numeric pseries::degree(const ex &s) const
 {
 	if (var.is_equal(s)) {
 		// Return last exponent
@@ -294,7 +295,7 @@ int pseries::degree(const ex &s) const
 			return 0;
 		int max_pow = std::numeric_limits<int>::min();
                 for (const auto & elem : seq) {
-			int pow = elem.rest.degree(s);
+			int pow = elem.rest.degree(s).to_int();
 			if (pow > max_pow)
 				max_pow = pow;
 		}
@@ -307,12 +308,12 @@ int pseries::degree(const ex &s) const
  *  series is examined termwise.  If s is the expansion variable but the
  *  expansion point is not zero the series is not expanded to find the degree.
  *  I.e.: (1-x) + (1-x)^2 + Order((1-x)^3) has ldegree(x) 1, not 0. */
-int pseries::ldegree(const ex &s) const
+numeric pseries::ldegree(const ex &s) const
 {
 	if (var.is_equal(s)) {
 		// Return first exponent
 		if (seq.size() != 0u)
-			return ex_to<numeric>((seq.begin())->coeff).to_int();
+			return ex_to<numeric>((seq.begin())->coeff);
 		else
 			return 0;
 	} else {
@@ -320,7 +321,7 @@ int pseries::ldegree(const ex &s) const
 			return 0;
 		int min_pow = std::numeric_limits<int>::max();
                 for (const auto & elem : seq) {
-			int pow = elem.rest.ldegree(s);
+			int pow = elem.rest.ldegree(s).to_int();
 			if (pow < min_pow)
 				min_pow = pow;
 		}
@@ -335,14 +336,16 @@ int pseries::ldegree(const ex &s) const
  *  If s is not the expansion variable, an attempt is made to convert the
  *  series to a polynomial and return the corresponding coefficient from
  *  there. */
-ex pseries::coeff(const ex &s, int n) const
+ex pseries::coeff(const ex &s, const ex & n) const
 {
 	if (var.is_equal(s)) {
 		if (seq.empty())
 			return _ex0;
 		
 		// Binary search in sequence for given power
-		numeric looking_for = numeric(n);
+                if (not is_exactly_a<numeric>(n))
+                        throw std::runtime_error("can't happen in pseries::coeff");
+		numeric looking_for = ex_to<numeric>(n);
 		int lo = 0, hi = seq.size() - 1;
 		while (lo <= hi) {
 			int mid = (lo + hi) / 2;
@@ -453,31 +456,6 @@ ex pseries::imag_part() const
 	return (new pseries(var==point, v))->setflag(status_flags::dynallocated);
 }
 
-ex pseries::eval_integ() const
-{
-	epvector *newseq = nullptr;
-	for (auto i=seq.begin(); i!=seq.end(); ++i) {
-		if (newseq != nullptr) {
-			newseq->push_back(expair(i->rest.eval_integ(), i->coeff));
-			continue;
-		}
-		ex newterm = i->rest.eval_integ();
-		if (!are_ex_trivially_equal(newterm, i->rest)) {
-			newseq = new epvector;
-			newseq->reserve(seq.size());
-			for (auto j=seq.begin(); j!=i; ++j)
-				newseq->push_back(*j);
-			newseq->push_back(expair(newterm, i->coeff));
-		}
-	}
-
-	ex newpoint = point.eval_integ();
-	if ((newseq != nullptr) || !are_ex_trivially_equal(newpoint, point))
-		return (new pseries(var==newpoint, *newseq))
-		       ->setflag(status_flags::dynallocated);
-	return *this;
-}
-
 ex pseries::subs(const exmap & m, unsigned options) const
 {
 	// If expansion variable is being substituted, convert the series to a
@@ -599,7 +577,7 @@ ex basic::series(const relational & r, int order, unsigned options) const
 	numeric fac = 1;
 	ex deriv = *this;
 	const ex co = deriv.subs(r, subs_options::no_pattern);
-	if (!co.is_zero()) {
+	if (not co.is_zero()) {
 		seq.push_back(expair(co, _ex0));
 	}
 
@@ -626,6 +604,15 @@ ex basic::series(const relational & r, int order, unsigned options) const
 }
 
 
+ex numeric::series(const relational & r, int order, unsigned options) const
+{
+	epvector seq;
+        if (not is_zero())
+                seq.push_back(expair(*this, _ex0));
+        seq.push_back(expair(Order(_ex1), numeric(order)));
+	return pseries(r, seq);
+}
+
 /** Implementation of ex::series() for symbols.
  *  @see ex::series */
 ex symbol::series(const relational & r, int order, unsigned options) const
@@ -790,10 +777,10 @@ ex pseries::mul_series(const pseries &other) const
 	
 	// Series multiplication
 	epvector new_seq;
-	const int a_max = degree(var);
-	const int b_max = other.degree(var);
-	const int a_min = ldegree(var);
-	const int b_min = other.ldegree(var);
+	const int a_max = degree(var).to_int();
+	const int b_max = other.degree(var).to_int();
+	const int a_min = ldegree(var).to_int();
+	const int b_min = other.ldegree(var).to_int();
 	const int cdeg_min = a_min + b_min;
 	int cdeg_max = a_max + b_max;
 	
@@ -869,7 +856,7 @@ ex mul::series(const relational & r, int order, unsigned options) const
 		int real_ldegree = 0;
 		bool flag_redo = false;
 		try {
-			real_ldegree = buf.expand().ldegree(sym-r.rhs());
+			real_ldegree = buf.expand().ldegree(sym-r.rhs()).to_int();
 		} catch (std::runtime_error) {}
 
 		if (real_ldegree == 0) {
@@ -879,13 +866,13 @@ ex mul::series(const relational & r, int order, unsigned options) const
 				int orderloop = 0;
 				do {
 					orderloop++;
-					real_ldegree = buf.series(r, orderloop, options).ldegree(sym);
+					real_ldegree = buf.series(r, orderloop, options).ldegree(sym).to_int();
 				} while (real_ldegree == orderloop);
 			} else {
 				// Here it is possible that buf does not have a ldegree, therefore
 				// check only if ldegree is negative, otherwise reconsider the case
 				// in the second round.
-				real_ldegree = buf.series(r, 0, options).ldegree(sym);
+				real_ldegree = buf.series(r, 0, options).ldegree(sym).to_int();
 				if (real_ldegree == 0)
 					flag_redo = true;
 			}
@@ -915,7 +902,7 @@ ex mul::series(const relational & r, int order, unsigned options) const
 			int orderloop = 0;
 			do {
 				orderloop++;
-				real_ldegree = buf.series(r, orderloop, options).ldegree(sym);
+				real_ldegree = buf.series(r, orderloop, options).ldegree(sym).to_int();
 			} while ((real_ldegree == orderloop)
 					&& ( factor*real_ldegree < degbound));
 			ldegrees[j] = factor * real_ldegree;
@@ -946,7 +933,7 @@ ex mul::series(const relational & r, int order, unsigned options) const
 			acc = ex_to<pseries>(acc.mul_series(ex_to<pseries>(term)));
 	}
 
-	return acc.mul_const(ex_to<numeric>(overall_coeff));
+	return acc.mul_const(overall_coeff);
 }
 
 
@@ -988,7 +975,7 @@ ex pseries::power_const(const numeric &p, int deg) const
 			return *this;
 	}
 	
-	const int ldeg = ldegree(var);
+	const int ldeg = ldegree(var).to_int();
 	if (!(p*ldeg).is_integer())
 		throw std::runtime_error("pseries::power_const(): trying to assemble a Puiseux series");
 
@@ -1132,12 +1119,12 @@ ex power::series(const relational & r, int order, unsigned options) const
 	ex eb = basis.expand();
 	int real_ldegree = 0;
 	if (eb.info(info_flags::rational_function))
-		real_ldegree = eb.ldegree(sym-r.rhs());
+		real_ldegree = eb.ldegree(sym-r.rhs()).to_int();
 	if (real_ldegree == 0) {
 		int orderloop = 0;
 		do {
 			orderloop++;
-			real_ldegree = basis.series(r, orderloop, options).ldegree(sym);
+			real_ldegree = basis.series(r, orderloop, options).ldegree(sym).to_int();
 		} while (real_ldegree == orderloop);
 	}
 
@@ -1184,61 +1171,6 @@ ex pseries::series(const relational & r, int order, unsigned options) const
 		return convert_to_poly().series(r, order, options);
 }
 
-ex integral::series(const relational & r, int order, unsigned options) const
-{
-	if (x.subs(r) != x)
-		throw std::logic_error("Cannot series expand wrt dummy variable");
-	
-	// Expanding integrant with r substituted taken in boundaries.
-	ex fseries = f.series(r, order, options);
-	epvector fexpansion;
-	fexpansion.reserve(fseries.nops());
-	for (size_t i=0; i<fseries.nops(); ++i) {
-		ex currcoeff = ex_to<pseries>(fseries).coeffop(i);
-		currcoeff = (currcoeff == Order(_ex1))
-			? currcoeff
-			: integral(x, a.subs(r), b.subs(r), currcoeff);
-		if (currcoeff != 0)
-			fexpansion.push_back(
-				expair(currcoeff, ex_to<pseries>(fseries).exponop(i)));
-	}
-
-	// Expanding lower boundary
-	ex result = (new pseries(r, fexpansion))->setflag(status_flags::dynallocated);
-	ex aseries = (a-a.subs(r)).series(r, order, options);
-	fseries = f.series(x == (a.subs(r)), order, options);
-	for (size_t i=0; i<fseries.nops(); ++i) {
-		ex currcoeff = ex_to<pseries>(fseries).coeffop(i);
-		if (is_order_function(currcoeff))
-			break;
-		ex currexpon = ex_to<pseries>(fseries).exponop(i);
-		int orderforf = order-ex_to<numeric>(currexpon).to_int()-1;
-		currcoeff = currcoeff.series(r, orderforf);
-		ex term = ex_to<pseries>(aseries).power_const(ex_to<numeric>(currexpon+1),order);
-		term = ex_to<pseries>(term).mul_const(ex_to<numeric>(-1/(currexpon+1)));
-		term = ex_to<pseries>(term).mul_series(ex_to<pseries>(currcoeff));
-		result = ex_to<pseries>(result).add_series(ex_to<pseries>(term));
-	}
-
-	// Expanding upper boundary
-	ex bseries = (b-b.subs(r)).series(r, order, options);
-	fseries = f.series(x == (b.subs(r)), order, options);
-	for (size_t i=0; i<fseries.nops(); ++i) {
-		ex currcoeff = ex_to<pseries>(fseries).coeffop(i);
-		if (is_order_function(currcoeff))
-			break;
-		ex currexpon = ex_to<pseries>(fseries).exponop(i);
-		int orderforf = order-ex_to<numeric>(currexpon).to_int()-1;
-		currcoeff = currcoeff.series(r, orderforf);
-		ex term = ex_to<pseries>(bseries).power_const(ex_to<numeric>(currexpon+1),order);
-		term = ex_to<pseries>(term).mul_const(ex_to<numeric>(1/(currexpon+1)));
-		term = ex_to<pseries>(term).mul_series(ex_to<pseries>(currcoeff));
-		result = ex_to<pseries>(result).add_series(ex_to<pseries>(term));
-	}
-
-	return result;
-}
-
 
 /** Compute the truncated series expansion of an expression.
  *  This function returns an expression containing an object of class pseries 
@@ -1261,8 +1193,32 @@ ex ex::series(const ex & r, int order, unsigned options) const
 	else
 		throw (std::logic_error("ex::series(): expansion point has unknown type"));
 	
-	e = bp->series(rel_, order, options);
-	return e;
+        if ((options & series_options::try_univariate_flint) != 0u
+                        and rel_.rhs().is_zero()) {
+                options &= ~series_options::try_univariate_flint;
+                symbolset syms = rel_.lhs().symbols();
+                if (syms.size() == 1
+                    and useries_can_handle(*this, *(syms.begin()))) {
+                        try {
+                                return GiNaC::useries(*this,
+                                                *(syms.begin()),
+                                                order,
+                                                options);
+                        }
+                        catch(flint_error) {
+                                ;
+                        }
+                }
+        }
+        e = bp->series(rel_, order, options);
+        if ((options & series_options::truncate) != 0u) {
+                epvector v = ex_to<pseries>(e).seq;
+                if (is_order_function((v.end()-1)->rest)) {
+                        v.erase(v.end()-1);
+                        return pseries(rel_, v);
+                }
+        }
+        return e;
 }
 
 } // namespace GiNaC
