@@ -253,11 +253,6 @@ ex relational::subs(const exmap & m, unsigned options) const
 		return subs_one_level(m, options);
 }
 
-ex relational::eval_ncmul(const exvector & v) const
-{
-	return lh.eval_ncmul(v);
-}
-
 // protected
 
 int relational::compare_same_type(const basic & other) const
@@ -344,7 +339,7 @@ long relational::calchash() const
 	v ^= lhash;
 
 	// store calculated hash value only if object is already evaluated
-	if ((flags & status_flags::evaluated) != 0u) {
+	if (is_evaluated()) {
 		setflag(status_flags::hash_calculated);
 		hashvalue = v;
 	}
