@@ -256,6 +256,8 @@ static funcmap_t& funcmap()
         return _funcmap;
 }
 
+static bool rational_ex_f;
+
 // Fast heuristic that rejects/accepts expressions for the fast
 // expansion via Flint. It can give false positives that must be
 // caught before Flint raises SIGABRT, because we want to use the
@@ -273,11 +275,13 @@ static bool unhandled_elements_in(ex the_ex, const symbol& symb)
                 return true;
         if (is_exactly_a<numeric>(the_ex))
                 return not (ex_to<numeric>(the_ex).is_mpz()
+                                or ex_to<numeric>(the_ex).is_long()
                                 or ex_to<numeric>(the_ex).is_mpq());
         if (is_exactly_a<symbol>(the_ex)) {
                 return (not ex_to<symbol>(the_ex).is_equal(symb));
         }
         if (is_exactly_a<function>(the_ex)) {
+                rational_ex_f = false;
                 function f = ex_to<function>(the_ex);
                 if (funcmap().find(f.get_serial()) == funcmap().end())
                         return true;
@@ -288,6 +292,8 @@ static bool unhandled_elements_in(ex the_ex, const symbol& symb)
         }
         if (is_exactly_a<power>(the_ex)) {
                 power pow = ex_to<power>(the_ex);
+                if (not is_exactly_a<numeric>(pow.op(1)))
+                        rational_ex_f = false;
                 return (unhandled_elements_in(pow.op(0), symb)
                      or unhandled_elements_in(pow.op(1), symb));
         }
@@ -297,7 +303,7 @@ static bool unhandled_elements_in(ex the_ex, const symbol& symb)
                         if (unhandled_elements_in(epseq.op(i), symb))
                                 return true;
                 }
-                if (unhandled_elements_in(epseq.op(epseq.nops()), symb))
+                if (unhandled_elements_in(epseq.get_overall_coeff(), symb))
                         return true;
                 return false;
         }
@@ -306,7 +312,22 @@ static bool unhandled_elements_in(ex the_ex, const symbol& symb)
 
 bool useries_can_handle(ex the_ex, const symbol& s)
 {
-        return (not unhandled_elements_in(the_ex, s));
+        rational_ex_f = true;
+        bool ok = (not unhandled_elements_in(the_ex, s));
+        if (ok) {
+                ex nd = the_ex.numer_denom();
+                try {
+                        long nhdeg = nd.op(0).degree(s).to_long();
+                        long nldeg = nd.op(0).ldegree(s).to_long();
+                        long dhdeg = nd.op(1).degree(s).to_long();
+                        long dldeg = nd.op(1).ldegree(s).to_long();
+                }
+                catch (conversion_error) {
+                        throw std::runtime_error("exponent too big");
+                }
+                catch (std::runtime_error) {}
+        }
+        return ok;
 }
 
 class ldegree_error : public std::runtime_error {
@@ -456,15 +477,14 @@ void add::useries(flint_series_t& fp, int order) const
                 }
                 fmpq_poly_add(fp.ft, fp.ft, fp1.ft);
         }
-        ex ovcoeff = op(nops());
-        if (not is_exactly_a<numeric>(ovcoeff))
-                throw std::runtime_error("non-numeric oc encountered");
-        numeric oc = ex_to<numeric>(ovcoeff);
+        const numeric& oc = overall_coeff;
         if (oc.is_zero())
                 return;
 
         flint_series_t fp1;
-        if (oc.is_mpz())
+        if (oc.is_long())
+                fmpq_poly_set_si(fp1.ft, oc.to_long());
+        else if (oc.is_mpz())
                 fmpq_poly_set_mpz(fp1.ft, oc.as_mpz());
         else
                 fmpq_poly_set_mpq(fp1.ft, oc.as_mpq());
@@ -482,14 +502,13 @@ void mul::useries(flint_series_t& fp, int order) const
                 fmpq_poly_mullow(fp.ft, fp.ft, fp1.ft, order+2);
                 fp.offset = newoff;
         }
-        ex ovcoeff = op(nops());
-        if (not is_exactly_a<numeric>(ovcoeff))
-                throw std::runtime_error("non-numeric oc encountered");
-        numeric oc = ex_to<numeric>(ovcoeff);
+        const numeric& oc = overall_coeff;
         if (oc.is_one())
                 return;
 
-        if (oc.is_mpz())
+        if (oc.is_long())
+                fmpq_poly_scalar_mul_si(fp.ft, fp.ft, oc.to_long());
+        else if (oc.is_mpz())
                 fmpq_poly_scalar_mul_mpz(fp.ft, fp.ft, oc.as_mpz());
         else
                 fmpq_poly_scalar_mul_mpq(fp.ft, fp.ft, oc.as_mpq());
@@ -589,7 +608,9 @@ void function::useries(flint_series_t& fp, int order) const
 
 void numeric::useries(flint_series_t& fp, int order) const
 {
-        if (is_mpz())
+        if (is_long())
+                fmpq_poly_set_si(fp.ft, to_long());
+        else if (is_mpz())
                 fmpq_poly_set_mpz(fp.ft, as_mpz());
         else
                 fmpq_poly_set_mpq(fp.ft, as_mpq());
